@@ -16,10 +16,11 @@ char *mystrdup (const char *str);
 char *mystrndup (const char *str, size_t n);
 #endif
 
+#define destroy_string(ps) do{ if(ps && *(ps)){free(*(ps)); *(ps)=NULL;}}while(0) 
+
 /* use it to set string as well as flags and deallocate memory
    referred to by the pointer */
-void set_string_value (char **target, char *string,
-		       unsigned long *set_flags, unsigned long flag);
+void set_string (char **target, char *string);
 
 #ifdef __cplusplus
 }
