@@ -523,6 +523,8 @@ const char *parse_argb_color( const char *color, CARD32 *pargb )
 						free( ptr );
 				}
 			}
+			if( !success && strlen(color) < 30 )
+				show_warning( "unable to parse color \"%s\"", color );
 			return success?&(color[i]):color;
 		}
 	}
@@ -654,7 +656,7 @@ stripcomments2 (char *source, char **comments )
 			curr = i - 1;
 		}
 	}
-	do{ --curr ; }while( isspace ((int)source[curr]) && curr > start);
+	do{ --curr ; }while( curr > start && isspace ((int)source[curr]));
 	source[curr+1] = '\0';
 	return &source[start];
 }
@@ -870,40 +872,65 @@ double parse_math(const char* str, char** endptr, double size) {
 	double total = 0;
 	char op = '+';
 	char minus = 0;
-	const char* startptr = str;
+	char logical_not = 0;
+/*	const char* startptr = str; */
 	if( str == NULL ) 
 		return 0 ;
-	while (*str) {
+
+	while (isspace((int)*str)) str++;
+	if( *str == '!' ) 
+	{
+		logical_not = 1;
+		++str ;
+	}else if( *str == '-' ) 
+	{
+		minus = 1 ;
+		++str ;
+	}
+
+	while (*str) 
+	{
 		while (isspace((int)*str)) str++;
-		if (!op) {
+		if (!op) 
+		{
 			if (*str == '+' || *str == '-' || *str == '*' || *str == '/') op = *str++;
 			else if (*str == '-') { minus = 1; str++; }
+			else if (*str == '!') { logical_not = 1; str++; }
 			else if (*str == ')') { str++; break; }
 			else break;
-		} else {
+		} else 
+		{
 			char* ptr;
 			double num;
-			if (*str == '(') num = parse_math(str + 1, &ptr, size);
-                      else if (*str == '$') {
-                              for (ptr = (char*)str + 1 ; *ptr && !isspace(*ptr) && *ptr != '+' && *ptr != '-' && *ptr != '*' && *ptr != '/' && *ptr != ')' ; ptr++);
-                              num = asxml_var_nget((char*)str + 1, ptr - (str + 1));
-                      }
-			else num = strtod(str, &ptr);
-			if (str != ptr) {
+			
+			if (*str == '(') 
+				num = parse_math(str + 1, &ptr, size);
+            else if (*str == '$') 
+			{
+            	for (ptr = (char*)str + 1 ; *ptr && !isspace(*ptr) && *ptr != '+' && *ptr != '-' && *ptr != '*' && *ptr != '!' && *ptr != '/' && *ptr != ')' ; ptr++);
+               	num = asxml_var_nget((char*)str + 1, ptr - (str + 1));
+            }else 
+				num = strtod(str, &ptr);
+			
+			if (str != ptr) 
+			{
 				if (*ptr == '%') num *= size / 100.0, ptr++;
 				if (minus) num = -num;
+				if (logical_not) num = !num;
+				
 				if (op == '+') total += num;
 				else if (op == '-') total -= num;
 				else if (op == '*') total *= num;
 				else if (op == '/' && num) total /= num;
-			} else break;
+			} else 
+				break;
 			str = ptr;
 			op = '\0';
-			minus = 0;
+			minus = logical_not = 0;
 		}
 	}
 	if (endptr) *endptr = (char*)str;
-	show_debug(__FILE__,"parse_math",__LINE__,"Parsed math [%s] with reference [%.2f] into number [%.2f].", startptr, size, total);
+/* 	show_debug(__FILE__,"parse_math",__LINE__,"Parsed math [%s] with reference [%.2f] into number [%.2f].", startptr, size, total); */
 	return total;
 }
 
@@ -1089,6 +1116,34 @@ get_string_list_item (char *ptr, char **item_start, char **item_end, char separa
 	return ptr;
 }
 
+Bool
+match_compound_string (char *string, char separator, char *what)
+{
+	if( string == NULL && what == NULL )
+		return True;
+
+	if( what != NULL )
+	{
+		register char *ptr = string ;
+		if( ptr ) 
+		{
+			int what_len = strlen(what);
+			while(*ptr)
+			{
+				char *item_start = NULL, *item_end = NULL;
+				int item_len ;
+				if ((ptr = get_string_list_item (ptr, &item_start, &item_end, separator)) == NULL)
+					break;
+				item_len = (item_end - item_start);
+				if( item_len == what_len ) 
+					if( strncmp(item_start, what, item_len ) == 0 ) 
+						return True;
+			}
+		}
+	}
+	return False ;
+}
+
 char        **
 compound_string2string_list (char *string, char separator, Bool duplicate, int *nitems_return)
 {
@@ -1196,12 +1251,12 @@ list2comma_string (char **list)
 }
 
 void
-destroy_string_list( char **list )
+destroy_string_list( char **list, int max_items )
 {
     if( list )
     {
-        register int i = -1;
-        while( list[++i] )  free( list[i] );
+        register int i = 0;
+        while( (i < max_items || max_items == 0) && list[i] )  { free( list[i] ); ++i; }
         free( list );
     }
 }
@@ -1254,3 +1309,31 @@ make_tricky_text( char *src )
     return trg;
 }
 
+char *interpret_ascii_string( const char *str )
+{
+	int src_i = 0, dst_i = 0; 
+	char *dst = safemalloc( (str?strlen(str):0)+1 );
+	
+	while( str[src_i] != '\0' ) 
+	{
+		if( str[src_i] == '\\' )
+		{
+			++src_i ; 
+			if( str[src_i] == 't' ) 
+				dst[dst_i++] = '\t' ;
+			else if( str[src_i] == 'n' ) 
+				dst[dst_i++] = '\n' ;
+			else if( str[src_i] == 'r' ) 
+				dst[dst_i++] = '\r' ;
+			else if( str[src_i] == '\\' ) 
+				dst[dst_i++] = '\\' ;
+			else if( str[src_i] == '\0' ) 
+				break;
+		}else
+			dst[dst_i++] = str[src_i] ;
+		
+		++src_i ;
+	}
+	dst[dst_i] = '\0' ;
+	return dst;
+}
