@@ -131,11 +131,13 @@ TermDef       StyleTerms[] = {
 	{TF_NO_MYNAME_PREPENDING, "FocusOnMap", 10, TT_FLAG, DATABASE_FocusOnMap_ID, NULL},
 	{TF_NO_MYNAME_PREPENDING, "NoFocusOnMap", 12, TT_FLAG, DATABASE_NoFocusOnMap_ID, NULL},
 	{TF_NO_MYNAME_PREPENDING, "LongLiving", 10, TT_FLAG, DATABASE_LongLiving_ID, NULL},
-	{TF_NO_MYNAME_PREPENDING, "ShortLiving", 12, TT_FLAG, DATABASE_ShortLiving_ID, NULL},
-	{TF_NO_MYNAME_PREPENDING, "IgnoreConfig", 13, TT_FLAG, DATABASE_IgnoreConfig_ID, NULL},
-	{TF_NO_MYNAME_PREPENDING, "HonorConfig", 12, TT_FLAG, DATABASE_HonorConfig_ID, NULL},
+	{TF_NO_MYNAME_PREPENDING, "ShortLiving", 11, TT_FLAG, DATABASE_ShortLiving_ID, NULL},
+	{TF_NO_MYNAME_PREPENDING, "IgnoreConfig", 12, TT_FLAG, DATABASE_IgnoreConfig_ID, NULL},
+	{TF_NO_MYNAME_PREPENDING, "HonorConfig", 11, TT_FLAG, DATABASE_HonorConfig_ID, NULL},
 	{TF_NO_MYNAME_PREPENDING, "UseCurrentViewport", 18, TT_FLAG, DATABASE_UseCurrentViewport_ID, NULL},
 	{TF_NO_MYNAME_PREPENDING, "UseAnyViewport", 14, TT_FLAG, DATABASE_UseAnyViewport_ID, NULL},
+	{TF_NO_MYNAME_PREPENDING, "IgnoreRestack", 13, TT_FLAG, DATABASE_IgnoreRestack_ID, NULL},
+	{TF_NO_MYNAME_PREPENDING, "HonorRestack", 12, TT_FLAG, DATABASE_HonorRestack_ID, NULL},
 	ASCF_DEFINE_KEYWORD(DATABASE,TF_NO_MYNAME_PREPENDING,Fullscreen,TT_FLAG,NULL),
 	ASCF_DEFINE_KEYWORD(DATABASE,TF_NO_MYNAME_PREPENDING,NoFullscreen,TT_FLAG,NULL),
 	ASCF_DEFINE_KEYWORD(DATABASE,TF_NO_MYNAME_PREPENDING,WindowOpacity,TT_INTEGER,NULL),
@@ -180,6 +182,7 @@ flag_options_xref StyleFlags[] = {
 	{STYLE_FOCUS_ON_MAP, 		DATABASE_FocusOnMap_ID, 		DATABASE_NoFocusOnMap_ID},
 	{STYLE_LONG_LIVING,			DATABASE_LongLiving_ID,			DATABASE_ShortLiving_ID},
 	{STYLE_IGNORE_CONFIG, 	    DATABASE_IgnoreConfig_ID, 	    DATABASE_HonorConfig_ID},
+	{STYLE_IGNORE_RESTACK, 	    DATABASE_IgnoreRestack_ID, 	    DATABASE_HonorRestack_ID},
 	{STYLE_CURRENT_VIEWPORT,    DATABASE_UseCurrentViewport_ID, 	DATABASE_UseAnyViewport_ID},
 	{STYLE_ICON, 				0 /* special case */ , 			DATABASE_NoIcon_ID},
 	{STYLE_STARTUP_DESK, 		0 /*special case */ , 			DATABASE_StartsAnywhere_ID},
@@ -439,6 +442,9 @@ ParseSingleStyle (FreeStorageElem * storage, name_list * style)
 			 set_string_value (&(style->icon_file), item.data.string, &(style->set_data_flags), STYLE_ICON);
 			 if (strlen (style->icon_file) <= 0)
 				 clear_flags (style->set_data_flags, STYLE_ICON);
+             /* regardles of weather the file was valid - indicate that app should have an icon !!! */
+			 set_flags (style->set_flags, STYLE_ICON);
+			 set_flags (style->flags, STYLE_ICON);
 			 break;
 		 case DATABASE_FocusStyle_ID:
 		 case DATABASE_UnfocusStyle_ID:
