@@ -58,8 +58,8 @@ flag_options_xref FeelFlagsXref[] = {
      /* placement : */
     {DontMoveOff           , FEEL_DontMoveOff_ID           , 0},
     {NoPPosition           , FEEL_NoPPosition_ID           , 0},
-    {SMART_PLACEMENT        , FEEL_SmartPlacement_ID        , 0},
-    {RandomPlacement       , FEEL_RandomPlacement_ID       , 0},
+//    {SMART_PLACEMENT        , FEEL_SmartPlacement_ID        , 0},
+//    {RandomPlacement       , FEEL_RandomPlacement_ID       , 0},
     {StubbornPlacement     , FEEL_StubbornPlacement_ID     , 0},
 //    {MenusHigh             , FEEL_MenusHigh_ID             , 0},
     {CenterOnCirculate     , FEEL_CenterOnCirculate_ID     , 0},
@@ -911,12 +911,12 @@ ParseThemeFile (const char *filename, char *myname)
         switch (pCurr->term->id)
 		{
             case FEEL_Function_ID           :
-                if( mystrncasecmp( item.data.string, THEME_INSTALL_FUNC_NAME, 12 ) == 0 )
+                if( mystrncasecmp( item.data.string, THEME_INSTALL_FUNC_NAME, sizeof(THEME_INSTALL_FUNC_NAME))== 0 )
                 {
                     if( config->install )
                         really_destroy_complex_func( config->install );
                      config->install = FreeStorage2ComplexFunction( pCurr, &item, NULL );
-                }else if( mystrncasecmp( item.data.string, THEME_APPLY_FUNC_NAME, 10 ) == 0 )
+                }else if( mystrncasecmp( item.data.string, THEME_APPLY_FUNC_NAME, sizeof(THEME_APPLY_FUNC_NAME) )== 0 )
                 {
                     if( config->apply )
                         really_destroy_complex_func( config->apply );
