@@ -494,6 +494,56 @@ asim_mystrndup (const char *str, size_t n)
 	return c;
 }
 
+int
+asim_mystrcasecmp (const char *s1, const char *s2)
+{
+	int          c1, c2;
+	register int i = 0 ;
+
+	if (s1 == NULL || s2 == NULL)
+		return (s1 == s2) ? 0 : ((s1==NULL)?1:-1);
+	while (s1[i])
+	{
+		/* in some BSD implementations, tolower(c) is not defined
+		 * unless isupper(c) is true */
+		c1 = s1[i];
+		if (isupper (c1))
+			c1 = tolower (c1);
+		c2 = s2[i];
+		if (isupper (c2))
+			c2 = tolower (c2);
+
+		++i ;
+		if (c1 != c2)
+			return (c1 - c2);
+	}
+	return -s2[i];
+}
+
+int
+asim_mystrncasecmp (const char *s1, const char *s2, size_t n)
+{
+	register int  c1, c2;
+	register int i = 0 ;
+
+	if (s1 == NULL || s2 == NULL)
+		return (s1 == s2) ? 0 : ((s1==NULL)?1:-1);
+	while( i < n )
+	{
+		c1 = s1[i], c2 = s2[i];
+		++i ;
+		if (c1==0)
+			return -c2;
+		if (isupper (c1))
+			c1 = tolower(c1);
+		if (isupper (c2))
+			c2 = tolower(c2);
+		if (c1 != c2)
+			return (c1 - c2);
+	}
+	return 0;
+}
+
 #ifdef X_DISPLAY_MISSING
 static int compare_xcolor_entries(const void *a, const void *b)
 {
@@ -921,6 +971,21 @@ void asim_flush_ashash_memory_pool()
 /************************************************************************/
 /* 	Some usefull implementations 					*/
 /************************************************************************/
+ASHashKey asim_pointer_hash_value (ASHashableValue value, ASHashKey hash_size)
+{
+    union
+    {
+        void *ptr;
+        ASHashKey key[2];
+    } mix;
+    register  ASHashKey key;
+
+    mix.ptr = (void*)value;
+    key = mix.key[0]^mix.key[1] ;
+    if( hash_size == 256 )
+		return (key>>4)&0x0FF;
+    return (key>>4) % hash_size;
+}
 
 /* case sensitive strings hash */
 ASHashKey
@@ -1215,6 +1280,8 @@ asim_my_scandir_ext ( const char *dirname, int (*filter_func) (const char *),
 	return n;
 }
 
+#endif /* #ifndef _WIN32 */
+
 /***************************************/
 /* from xml.c                          */
 /***************************************/
@@ -1271,7 +1338,7 @@ asim_asxml_var_get(const char* name)
     return hdata.i;
 }
 
-int
+static int
 asim_asxml_var_nget(char* name, int n) {
       int value;
       char oldc = name[n];
@@ -1625,7 +1692,7 @@ void asim_reset_xml_buffer( ASXmlBuffer *xb )
 {
 	if( xb ) 
 	{
-		xb->used = 0 ; 
+		xb->current = xb->used = 0 ; 
 		xb->state = ASXML_Start	 ;
 		xb->level = 0 ;
 		xb->verbatim = False ;
@@ -1635,19 +1702,104 @@ void asim_reset_xml_buffer( ASXmlBuffer *xb )
 	}		  
 }	 
 
-
 void 
-asim_add_xml_buffer_chars( ASXmlBuffer *xb, char *tmp, int len )
+asim_free_xml_buffer_resources (ASXmlBuffer *xb)
+{
+	if (xb && xb->buffer)
+	{
+		free (xb->buffer);
+		xb->allocated = xb->current = xb->used = 0 ; 
+		xb->buffer = NULL;
+	}
+}
+
+static inline void
+realloc_xml_buffer( ASXmlBuffer *xb, int len )
 {
 	if( xb->used + len > xb->allocated ) 
 	{	
 		xb->allocated = xb->used + (((len>>11)+1)<<11) ;	  
 		xb->buffer = realloc( xb->buffer, xb->allocated );
 	}
+}
+
+void 
+asim_add_xml_buffer_chars( ASXmlBuffer *xb, char *tmp, int len )
+{
+	realloc_xml_buffer (xb, len);
 	memcpy( &(xb->buffer[xb->used]), tmp, len );
 	xb->used += len ;
 }
 
+static void 
+add_xml_buffer_spaces( ASXmlBuffer *xb, int len )
+{
+	if (len > 0)
+	{
+		realloc_xml_buffer (xb, len);
+		memset( &(xb->buffer[xb->used]), ' ', len );
+		xb->used += len ;
+	}
+}
+
+static void 
+add_xml_buffer_open_tag( ASXmlBuffer *xb, xml_elem_t *tag )
+{
+	int tag_len = strlen (tag->tag);
+	int parm_len = 0;
+	xml_elem_t* parm = NULL ; 
+	
+	if (tag->parm)
+	{
+		xml_elem_t *t = parm = xml_parse_parm(tag->parm, NULL);
+		while (t)
+		{
+			parm_len += 1 + strlen(t->tag) + 1 + 1 + strlen(t->parm) + 1;
+			t = t->next;
+		}
+	}
+	realloc_xml_buffer (xb, 1+tag_len+1+parm_len+2);
+	xb->buffer[(xb->used)++] = '<';
+	memcpy (&(xb->buffer[xb->used]), tag->tag, tag_len);
+	xb->used += tag_len ;
+
+	while (parm) 
+	{
+		xml_elem_t* p = parm->next;
+		int len;
+		xb->buffer[(xb->used)++] = ' ';
+		for (len = 0 ; parm->tag[len] ; ++len)
+			xb->buffer[xb->used+len] = parm->tag[len];
+		xb->used += len ;
+		xb->buffer[(xb->used)++] = '=';
+		xb->buffer[(xb->used)++] = '\"';
+		for (len = 0 ; parm->parm[len] ; ++len)
+			xb->buffer[xb->used+len] = parm->parm[len];
+		xb->used += len ;
+		xb->buffer[(xb->used)++] = '\"';
+		free(parm->tag);
+		free(parm->parm);
+		free(parm);
+		parm = p;
+	}
+
+	if (tag->child == NULL)
+		xb->buffer[(xb->used)++] = '/';
+	xb->buffer[(xb->used)++] = '>';
+}
+
+static void 
+add_xml_buffer_close_tag( ASXmlBuffer *xb, xml_elem_t *tag )
+{
+	int tag_len = strlen (tag->tag);
+	realloc_xml_buffer (xb, tag_len+3);
+	xb->buffer[(xb->used)++] = '<';
+	xb->buffer[(xb->used)++] = '/';
+	memcpy (&(xb->buffer[xb->used]), tag->tag, tag_len);
+	xb->used += tag_len ;
+	xb->buffer[(xb->used)++] = '>';
+}
+
 int 
 asim_spool_xml_tag( ASXmlBuffer *xb, char *tmp, int len )
 {
@@ -1861,5 +2013,92 @@ asim_spool_xml_tag( ASXmlBuffer *xb, char *tmp, int len )
 	return (i==0)?1:i;
 }	   
 
+/* reverse transformation - put xml tags into a buffer */
+Bool 
+asim_xml_tags2xml_buffer( xml_elem_t *tags, ASXmlBuffer *xb, int tags_count, int depth)
+{
+	Bool new_line = False; 
 
-#endif
+	while (tags && tags_count != 0) /* not a bug - negative tags_count means unlimited !*/
+	{
+		if (tags->tag_id == XML_CDATA_ID || !strcmp(tags->tag, cdata_str)) 
+		{
+			/* TODO : add handling for cdata with quotes, amps and gt, lt */
+			add_xml_buffer_chars( xb, tags->parm, strlen(tags->parm));
+		}else 
+		{
+			if (depth >= 0 && (tags->child != NULL || tags->next != NULL)) 
+			{
+				add_xml_buffer_chars( xb, "\n", 1);
+				add_xml_buffer_spaces( xb, depth*2);
+				new_line = True ;	  
+			}
+			add_xml_buffer_open_tag( xb, tags);
+
+			if (tags->child) 
+			{
+				if( xml_tags2xml_buffer( tags->child, xb, -1, (depth < 0)?-1:depth+1 ))
+				{
+					if (depth >= 0)
+					{
+						add_xml_buffer_chars( xb, "\n", 1);
+						add_xml_buffer_spaces( xb, depth*2);
+					}
+				}
+				add_xml_buffer_close_tag( xb, tags);
+			}
+		}		
+		tags = tags->next;
+		--tags_count;
+	}
+	return new_line;
+}
+
+void asim_xml_print(xml_elem_t* root) 
+{
+	ASXmlBuffer xb;
+	memset( &xb, 0x00, sizeof(xb));
+	xml_tags2xml_buffer( root, &xb, -1, 0);
+	add_xml_buffer_chars( &xb, "\0", 1 );
+	printf ("%s", xb.buffer);
+	free_xml_buffer_resources (&xb);
+}
+
+
+xml_elem_t *
+asim_format_xml_buffer_state (ASXmlBuffer *xb)
+{
+	xml_elem_t *state_xml = NULL; 
+	if (xb->state < 0) 
+	{
+		state_xml = xml_elem_new();
+		state_xml->tag = mystrdup("error");
+		state_xml->parm = safemalloc (64);
+		sprintf(state_xml->parm, "code=%d level=%d tag_count=%d", xb->state, xb->level ,xb->tags_count );
+		state_xml->child = create_CDATA_tag();
+		switch( xb->state ) 
+		{
+			case ASXML_BadStart : state_xml->child->parm = mystrdup("Text encountered before opening tag bracket - not XML format"); break;
+			case ASXML_BadTagName : state_xml->child->parm = mystrdup("Invalid characters in tag name" );break;
+			case ASXML_UnexpectedSlash : state_xml->child->parm = mystrdup("Unexpected '/' encountered");break;
+			case ASXML_UnmatchedClose : state_xml->child->parm = mystrdup("Closing tag encountered without opening tag" );break;
+			case ASXML_BadAttrName : state_xml->child->parm = mystrdup("Invalid characters in attribute name" );break;
+			case ASXML_MissingAttrEq : state_xml->child->parm = mystrdup("Attribute name not followed by '=' character" );break;
+			default:
+				state_xml->child->parm = mystrdup("Premature end of the input");break;
+		}
+	}else if (xb->state == ASXML_Start)
+	{
+		if (xb->tags_count > 0)
+		{
+			state_xml = xml_elem_new();
+			state_xml->tag = mystrdup("success");
+			state_xml->parm = safemalloc(64);
+			sprintf(state_xml->parm, "tag_count=%d level=%d", xb->tags_count, xb->level );
+		}
+	}else
+	{
+		/* TODO */
+	}
+	return state_xml;
+}
