@@ -1048,7 +1048,7 @@ load_glyph_freetype( ASFont *font, ASGlyph *asg, int glyph, UNICODE_CHAR uc )
 #if 0			
 			asg->step = bmap->width+face->glyph->bitmap_left ;
 #else
-			asg->step = face->glyph->advance.x>>6 ;
+			asg->step = (short)face->glyph->advance.x>>6 ;
 #endif
 				
 		/* we only want to keep lead if it was negative */
@@ -1445,7 +1445,7 @@ goto_tab_stop( ASTextAttributes *attr, unsigned int space_size, unsigned int lin
 	do{ if( (prev_gid) != 0 && font->type == ASF_Freetype && get_flags(font->flags, ASF_Monospaced|ASF_HasKerning) == ASF_HasKerning ) { \
 		FT_Vector delta; \
 		FT_Get_Kerning( font->ft_face, (prev_gid), (this_gid), FT_KERNING_DEFAULT, &delta );\
-		(var) = delta.x >> 6; \
+		(var) = (short)(delta.x >> 6); \
 	}}while(0)
 #else
 #define GET_KERNING(var,prev_gid,this_gid)	do{(var)=0;}while(0)	  
@@ -1578,13 +1578,13 @@ get_text_glyph_map( const char *text, ASFont *font, ASGlyphMap *map, ASTextAttri
 	map->glyphs = safecalloc( map->glyphs_num, sizeof(ASGlyph*));
 	map->x_kerning = safecalloc( map->glyphs_num, sizeof(short));
 
-	if( attr->char_type == ASCT_Char )
-		line_count = fill_text_glyph_map_Char( text, font, map, attr, space_size, offset_3d_x );
-	else if( attr->char_type == ASCT_UTF8 )
+	if( attr->char_type == ASCT_UTF8 )
 		line_count = fill_text_glyph_map_UTF8( text, font, map, attr, space_size, offset_3d_x );
 	else if( attr->char_type == ASCT_Unicode )
 		line_count = fill_text_glyph_map_Unicode( (UNICODE_CHAR*)text, font, map, attr, space_size, offset_3d_x );
-
+	else /* assuming attr->char_type == ASCT_Char by default */
+		line_count = fill_text_glyph_map_Char( text, font, map, attr, space_size, offset_3d_x );
+	
     map->height = line_count * (font->max_height+offset_3d_y) - font->spacing_y;
 
 	if( map->height <= 0 )
