@@ -110,44 +110,6 @@ Bool asimage_use_mmx = True;
 #else
 Bool asimage_use_mmx = False;
 #endif
-/* ********************************************************************/
-/* initialization routines 											  */
-/* *********************************************************************/
-/* the following is depreciated - we moved to use intrinsics 	!!!!!! */
-/* ************************** MMX **************************************/
-/*inline extern*/
-int mmx_init(void)
-{
-int mmx_available = 0;
-#ifdef HAVE_MMX
-	int ebx_save;
- 	asm volatile (
-                      "movl %%ebx, %1\n\t"
-                      "movl $1, %%eax\n\t"
-                      "cpuid\n\t"
-                      "andl $0x800000, %%edx\n\t"	
-                      "movl %1, %%ebx\n\t"
-                      "movl %%edx, %0\n\t"
-                      : "=m" (mmx_available), "=m" (ebx_save)
-                      : /* no input */
-                      : "eax", "ecx", "edx"
-                     );
-#endif
-	return mmx_available;
-}
-
-int mmx_off(void)
-{
-#ifdef HAVE_MMX
-	__asm__ __volatile__ (
-                      /* exit mmx state : */
-                      "emms \n\t"
-                      : /* no output */
-                      : /* no input  */
-			  );
-#endif
-	return 0;
-}
 
 /* *********************   ASImage  ************************************/
 void
@@ -244,17 +206,20 @@ asimage_replace (ASImage *im, ASImage *from)
 		{
 			int ref_count = im->ref_count ;
 			ASImageManager *imageman = im->imageman ;
-			char *name = im->name ; 
+			char *name = im->name ;
+			ASFlagType  saved_flags = im->flags & (ASIM_NAME_IS_FILENAME|ASIM_NO_COMPRESSION) ;
 
 			im->name = NULL ; 
 			asimage_init (im, True);
 
 			memcpy( im, from, sizeof(ASImage) );
+			/* Assume : from->name == NULL as from->imageman == NULL (see above ) */
 			memset( from, 0x00, sizeof(ASImage) );		
 
 			im->ref_count = ref_count ; 
 			im->imageman = imageman ;
 			im->name = name ;
+			set_flags( im->flags, saved_flags );
 
 			return True ;
 		}
@@ -297,6 +262,8 @@ void
 destroy_asimage( ASImage **im )
 {
 	if( im )
+	{
+
 		if( *im && !AS_ASSERT_NOTVAL((*im)->imageman,NULL))
 		{
 #ifdef TRACK_ASIMAGES
@@ -307,7 +274,13 @@ destroy_asimage( ASImage **im )
 			(*im)->magic = 0;
 			free( *im );
 			*im = NULL ;
+		}else if( *im )
+		{
+	        show_error( "Failed to destroy ASImage %p:", *im );
+			print_asimage_func (AS_HASHABLE(*im));
 		}
+
+	}
 }
 
 void print_asimage_func (ASHashableValue value)
@@ -535,13 +508,19 @@ dup_asimage( ASImage* im )
 {
 	if( !AS_ASSERT(im) )
 		if( AS_ASSERT_NOTVAL(im->magic,MAGIC_ASIMAGE) )
+		{
 			im = NULL ;
+			show_error( "ASImage %p has invalid magic number - discarding!", im );
+		}
 
 	if( !AS_ASSERT(im) && !AS_ASSERT(im->imageman) )
 	{
 /*		fprintf( stderr, __FUNCTION__" on image %p ref_count = %d\n", im, im->ref_count ); */
 		im->ref_count++ ;
 		return im;
+	}else if( im ) 
+	{
+		show_debug( __FILE__, "dup_asimage", __LINE__, "Attempt to duplicate ASImage %p that is not tracked by any image manager!", im );
 	}
 	return NULL ;
 }
@@ -839,6 +818,35 @@ asimage_add_line (ASImage * im, ColorPart color, register CARD32 * data, unsigne
 	return im->width;
 }
 
+size_t
+asimage_add_line_bgra (ASImage * im, register CARD32 * data, unsigned int y)
+{
+	if (AS_ASSERT(im) )
+		return 0;
+	if (y >= im->height)
+		return 0;
+	if( im->channels[IC_ALPHA][y] ) 
+		forget_data( NULL, im->channels[IC_ALPHA][y] ); 
+	im->channels[IC_ALPHA][y] = store_data( NULL, (CARD8*)data, im->width*4, 
+	                                        ASStorage_24BitShift|ASStorage_Masked|
+											ASStorage_RLEDiffCompress|ASStorage_32Bit, 0);
+	if( im->channels[IC_RED][y] ) 
+		forget_data( NULL, im->channels[IC_RED][y] ); 
+	im->channels[IC_RED][y] = store_data( NULL, (CARD8*)data, im->width*4, 
+	                                        ASStorage_16BitShift|ASStorage_Masked|
+											ASStorage_RLEDiffCompress|ASStorage_32Bit, 0);
+	if( im->channels[IC_GREEN][y] ) 
+		forget_data( NULL, im->channels[IC_GREEN][y] ); 
+	im->channels[IC_GREEN][y] = store_data( NULL, (CARD8*)data, im->width*4, 
+	                                        ASStorage_8BitShift|ASStorage_Masked|
+											ASStorage_RLEDiffCompress|ASStorage_32Bit, 0);
+	if( im->channels[IC_BLUE][y] ) 
+		forget_data( NULL, im->channels[IC_BLUE][y] ); 
+	im->channels[IC_BLUE][y] = store_data( NULL, (CARD8*)data, im->width*4, 
+	                                        ASStorage_Masked|
+											ASStorage_RLEDiffCompress|ASStorage_32Bit, 0);
+	return im->width;
+}
 
 unsigned int
 asimage_print_line (ASImage * im, ColorPart color, unsigned int y, unsigned long verbosity)
@@ -889,7 +897,7 @@ LOCAL_DEBUG_CALLER_OUT( "im->width = %d, color = %d, y = %d, skip = %d, out_widt
         {
             int z = -1 ;
             while( ++z < i )
-                fprintf( stderr, "%lX ", to_buf[z] );
+                fprintf( stderr, "%lX ", (unsigned long)to_buf[z] );
             fprintf( stderr, "\n");
 
         }
@@ -986,7 +994,7 @@ asimage_compare_line (ASImage *im, ColorPart color, CARD32 *to_buf, CARD32 *tmp,
 		if( tmp[i] != to_buf[i] )
 		{
 			if( verbose )
-				show_error( "line %d, component %d differ at offset %d ( 0x%lX(compresed) != 0x%lX(orig) )\n", y, color, i, tmp[i], to_buf[i] );
+				show_error( "line %d, component %d differ at offset %d ( 0x%lX(compresed) != 0x%lX(orig) )\n", y, color, i, (unsigned long)tmp[i], (unsigned long)to_buf[i] );
 			return False ;
 		}
 	return True;
