@@ -195,30 +195,30 @@ ASFontManager *create_generic_fontman(Display *dpy, const char *path)
 }
 
 ASImage *
-compose_asimage_xml(ASVisual *asv, ASImageManager *imman, ASFontManager *fontman, char *doc_str, ASFlagType flags, int verbose, Window display_win, const char *path)
+compose_asimage_xml_from_doc(ASVisual *asv, ASImageManager *imman, ASFontManager *fontman, xml_elem_t* doc, ASFlagType flags, int verbose, Window display_win, const char *path, int target_width, int target_height)
 {
+	/* Build the image(s) from the xml document structure. */
 	ASImage* im = NULL;
-	xml_elem_t* doc;
 	ASImageManager *my_imman = imman, *old_as_xml_imman = _as_xml_image_manager ;
 	ASFontManager  *my_fontman = fontman, *old_as_xml_fontman = _as_xml_font_manager ;
 	int my_imman_curr_dir_path_idx = MAX_SEARCH_PATHS ;
 
-    asxml_var_init();
-
-	doc = xml_parse_doc(doc_str, NULL);
-#if (HAVE_AFTERBASE_FLAG==1)
-	if (verbose > 1) 
-	{
-		xml_print(doc);
-		fprintf(stderr, "\n");
-	}
-#endif
-
-	/* Build the image(s) from the xml document structure. */
 	if (doc)
 	{
+		int old_target_width = -1;
+		int old_target_height = -1;
 		xml_elem_t* ptr;
 		Bool local_dir_included = False ;
+
+	    asxml_var_init();
+#if (HAVE_AFTERBASE_FLAG==1)
+		if (verbose > 1) 
+		{
+			xml_print(doc);
+			fprintf(stderr, "\n");
+		}
+#endif
+
 		if( my_imman == NULL )
 		{	
 			if( _as_xml_image_manager == NULL )
@@ -248,15 +248,40 @@ compose_asimage_xml(ASVisual *asv, ASImageManager *imman, ASFontManager *fontman
 				_as_xml_font_manager = create_generic_fontman( asv->dpy, path );
 			my_fontman = _as_xml_font_manager ;
 		}
-		
+
+		/* save old target size to be restored at the end */		
+		old_target_width = asxml_var_get(ASXMLVAR_TargetWidth);
+		old_target_height = asxml_var_get(ASXMLVAR_TargetHeight);
+		/* set current target size */		
+		asxml_var_insert(ASXMLVAR_TargetWidth, target_width);
+		asxml_var_insert(ASXMLVAR_TargetHeight, target_height);
+
 		for (ptr = doc->child ; ptr ; ptr = ptr->next) {
 			ASImage* tmpim = build_image_from_xml(asv, my_imman, my_fontman, ptr, NULL, flags, verbose, display_win);
 			if (tmpim && im) safe_asimage_destroy(im);
 			if (tmpim) im = tmpim;
 		}
+		if (im && (target_width > 0 || target_height > 0) )
+		  {
+			int scale_width = (target_width>0)?target_width:im->width;
+			int scale_height = (target_height>0)?target_height:im->height;
+			if (im->width != scale_width || im->height != scale_height)
+			  {
+			  	ASImage *tmp = scale_asimage( asv, im, scale_width, scale_height, ASA_ASImage, 100, ASIMAGE_QUALITY_DEFAULT );
+				if (tmp != NULL)
+				  {
+				  	safe_asimage_destroy(im);
+					im = tmp;
+				  }						
+			  }
+		  }
+		/* restore old target size to be restored at the end */		
+		asxml_var_insert(ASXMLVAR_TargetWidth, old_target_width);
+		asxml_var_insert(ASXMLVAR_TargetHeight, old_target_height);
+
 LOCAL_DEBUG_OUT( "result im = %p, im->imman	= %p, my_imman = %p, im->magic = %8.8lX", im, im?im->imageman:NULL, my_imman, im?im->magic:0 );
 		
-		if( my_imman_curr_dir_path_idx < MAX_SEARCH_PATHS ) 
+		if( my_imman_curr_dir_path_idx < MAX_SEARCH_PATHS && my_imman->search_path[my_imman_curr_dir_path_idx]) 
 		{
 			free(my_imman->search_path[my_imman_curr_dir_path_idx]);
 			my_imman->search_path[my_imman_curr_dir_path_idx] = NULL ;			
@@ -275,13 +300,32 @@ LOCAL_DEBUG_OUT( "result im = %p, im->imman	= %p, my_imman = %p, im->magic = %8.
 		_as_xml_image_manager = old_as_xml_imman   ;
 		_as_xml_font_manager =  old_as_xml_fontman ;
 		
-		/* Delete the xml. */
-		xml_elem_delete(NULL, doc);
 	}
 	LOCAL_DEBUG_OUT( "returning im = %p, im->imman	= %p, im->magic = %8.8lX", im, im?im->imageman:NULL, im?im->magic:0 );
 	return im;
 }
 
+ASImage *
+compose_asimage_xml_at_size(ASVisual *asv, ASImageManager *imman, ASFontManager *fontman, char *doc_str, ASFlagType flags, int verbose, Window display_win, const char *path, int target_width, int target_height)
+{
+	xml_elem_t* doc = xml_parse_doc(doc_str, NULL);
+	ASImage *im = compose_asimage_xml_from_doc(asv, imman, fontman, doc, flags, verbose, display_win, path, target_width, target_height);
+	if (doc)
+		xml_elem_delete(NULL, doc);
+	return im;
+}
+
+inline ASImage *
+compose_asimage_xml(ASVisual *asv, ASImageManager *imman, ASFontManager *fontman, char *doc_str, ASFlagType flags, int verbose, Window display_win, const char *path)
+{
+	xml_elem_t* doc = xml_parse_doc(doc_str, NULL);
+	ASImage *im = compose_asimage_xml_from_doc(asv, imman, fontman, doc, flags, verbose, display_win, path, -1, -1);
+	if (doc)
+		xml_elem_delete(NULL, doc);
+	return im;
+}
+
+
 Bool save_asimage_to_file(const char *file2bsaved, ASImage *im,
 	           const char *strtype,
 			   const char *compress,
@@ -860,7 +904,7 @@ handle_asxml_tag_composite( ASImageXMLState *state, xml_elem_t* doc, xml_elem_t*
  * NAME
  * img - load image from the file.
  * SYNOPSIS
- * <img id="new_img_id" src=filename/>
+ * <img id="new_img_id" src="filename"/>
  * ATTRIBUTES
  * id     Optional.  Image will be given this name for future reference.
  * src    Required.  The filename (NOT URL) of the image file to load.
@@ -889,7 +933,7 @@ handle_asxml_tag_img( ASImageXMLState *state, xml_elem_t* doc, xml_elem_t* parm,
 			result = pixmap2asimage(state->asv, rp, 0, 0, width, height, 0xFFFFFFFF, False, 100);
 			if( dst_width == 0 ) dst_width = width ; 
 			if( dst_height == 0 ) dst_height = height ; 
-			if( dst_width != width || dst_height != height ) 
+			if( dst_width != (int)width || dst_height != (int)height ) 
 			{
 				ASImage *tmp = scale_asimage( NULL, result, dst_width, dst_height, ASA_ASImage, 100, ASIMAGE_QUALITY_DEFAULT );
 				if( tmp ) 
@@ -953,9 +997,9 @@ handle_asxml_tag_recall( ASImageXMLState *state, xml_elem_t* doc, xml_elem_t* pa
 
 /****** libAfterImage/asimagexml/release
  * NAME
- * release - release(destroy if possible) previously generated and named image by its id.
+ * release - release (destroy if possible) previously generated and named image by its id.
  * SYNOPSIS
- * <release srcid="image_id">
+ * <release srcid="image_id"/>
  * ATTRIBUTES
  * srcid    Required.  An image ID defined with the "id" parameter for
  *          any previously created image.
@@ -980,7 +1024,7 @@ handle_asxml_tag_release( ASImageXMLState *state, xml_elem_t* doc, xml_elem_t* p
  * NAME
  * color - defines symbolic name for a color and set of variables.
  * SYNOPSIS
- * <color name="sym_name" domain="var_domain" argb=colorvalue/>
+ * <color name="sym_name" domain="var_domain" argb="colorvalue"/>
  * ATTRIBUTES
  * name   Symbolic name for the color value, to be used to refer to that color.
  * argb   8 characters hex definition of the color or other symbolic color name.
@@ -1790,7 +1834,7 @@ handle_asxml_tag_rotate( ASImageXMLState *state, xml_elem_t* doc, xml_elem_t* pa
  * NAME
  * scale - scale image to arbitrary size
  * SYNOPSIS
- * <scale id="new_id" ref_id="other_imag" src_x="pixels"  src_y="pixels" 
+ * <scale id="new_id" refid="other_imag" src_x="pixels"  src_y="pixels" 
  *        src_width="pixels" src_height="pixels" 
  *        width="pixels" height="pixels">
  * ATTRIBUTES
@@ -1829,10 +1873,10 @@ handle_asxml_tag_scale( ASImageXMLState *state, xml_elem_t* doc, xml_elem_t* par
 	LOCAL_DEBUG_OUT("doc = %p, parm = %p, imtmp = %p, width = %d, height = %d", doc, parm, imtmp, width, height ); 
 	for (ptr = parm ; ptr ; ptr = ptr->next) 
 	{
-		if (!strcmp(ptr->tag, "src_x")) 		src_x = parse_math(ptr->parm, NULL, width);
-		else if (!strcmp(ptr->tag, "src_y")) 	src_y = parse_math(ptr->parm, NULL, width);
-		else if (!strcmp(ptr->tag, "src_width")) 	src_width = parse_math(ptr->parm, NULL, width);
-		else if (!strcmp(ptr->tag, "src_height")) 	src_height = parse_math(ptr->parm, NULL, width);
+		if (!strcmp(ptr->tag, "src_x")) 		src_x = (int)parse_math(ptr->parm, NULL, width);
+		else if (!strcmp(ptr->tag, "src_y")) 	src_y = (int)parse_math(ptr->parm, NULL, width);
+		else if (!strcmp(ptr->tag, "src_width")) 	src_width = (int)parse_math(ptr->parm, NULL, width);
+		else if (!strcmp(ptr->tag, "src_height")) 	src_height = (int)parse_math(ptr->parm, NULL, width);
 	}	
 	if( state->verbose > 1 )
 		show_progress("Scaling image to [%dx%d].", width, height);
@@ -1890,10 +1934,10 @@ handle_asxml_tag_slice( ASImageXMLState *state, xml_elem_t* doc, xml_elem_t* par
 	LOCAL_DEBUG_OUT("doc = %p, parm = %p, imtmp = %p, width = %d, height = %d", doc, parm, imtmp, width, height ); 
 	for (ptr = parm ; ptr ; ptr = ptr->next) 
 	{
-		if (!strcmp(ptr->tag, "x_start")) 		x_start = parse_math(ptr->parm, NULL, width);
-		else if (!strcmp(ptr->tag, "x_end")) 	x_end = parse_math(ptr->parm, NULL, width);
-		else if (!strcmp(ptr->tag, "y_start")) 	y_start = parse_math(ptr->parm, NULL, height);
-		else if (!strcmp(ptr->tag, "y_end")) 	y_end = parse_math(ptr->parm, NULL, height);
+		if (!strcmp(ptr->tag, "x_start")) 		x_start = (int)parse_math(ptr->parm, NULL, width);
+		else if (!strcmp(ptr->tag, "x_end")) 	x_end = (int)parse_math(ptr->parm, NULL, width);
+		else if (!strcmp(ptr->tag, "y_start")) 	y_start = (int)parse_math(ptr->parm, NULL, height);
+		else if (!strcmp(ptr->tag, "y_end")) 	y_end = (int)parse_math(ptr->parm, NULL, height);
 		else if (!strcmp(ptr->tag, "scale")) 	scale = (ptr->parm[0] == '1');
 	}
 
@@ -1937,8 +1981,8 @@ handle_asxml_tag_crop( ASImageXMLState *state, xml_elem_t* doc, xml_elem_t* parm
 	int srcx = 0, srcy = 0;
 	LOCAL_DEBUG_OUT("doc = %p, parm = %p, imtmp = %p, width = %d, height = %d", doc, parm, imtmp, width, height ); 
 	for (ptr = parm ; ptr ; ptr = ptr->next) {
-		if (!strcmp(ptr->tag, "srcx")) srcx = parse_math(ptr->parm, NULL, width);
-		else if (!strcmp(ptr->tag, "srcy")) srcy = parse_math(ptr->parm, NULL, height);
+		if (!strcmp(ptr->tag, "srcx")) srcx = (int)parse_math(ptr->parm, NULL, width);
+		else if (!strcmp(ptr->tag, "srcy")) srcy = (int)parse_math(ptr->parm, NULL, height);
 		else if (!strcmp(ptr->tag, "tint")) parse_argb_color(ptr->parm, &tint);
 	}
 	if( state->verbose > 1 )
@@ -1990,8 +2034,8 @@ handle_asxml_tag_tile( ASImageXMLState *state, xml_elem_t* doc, xml_elem_t* parm
 	char *complement_str = NULL ;
 	LOCAL_DEBUG_OUT("doc = %p, parm = %p, imtmp = %p, width = %d, height = %d", doc, parm, imtmp, width, height ); 
 	for (ptr = parm ; ptr ; ptr = ptr->next) {
-		if (!strcmp(ptr->tag, "x_origin")) xorig = parse_math(ptr->parm, NULL, width);
-		else if (!strcmp(ptr->tag, "y_origin")) yorig = parse_math(ptr->parm, NULL, height);
+		if (!strcmp(ptr->tag, "x_origin")) xorig = (int)parse_math(ptr->parm, NULL, width);
+		else if (!strcmp(ptr->tag, "y_origin")) yorig = (int)parse_math(ptr->parm, NULL, height);
 		else if (!strcmp(ptr->tag, "tint")) parse_argb_color(ptr->parm, &tint);
 		else if (!strcmp(ptr->tag, "complement")) complement_str = ptr->parm;
 	}
@@ -2085,8 +2129,8 @@ handle_asxml_tag_hsv( ASImageXMLState *state, xml_elem_t* doc, xml_elem_t* parm,
 	LOCAL_DEBUG_OUT("doc = %p, parm = %p, imtmp = %p, width = %d, height = %d", doc, parm, imtmp, width, height ); 
 	for (ptr = parm ; ptr ; ptr = ptr->next) 
 	{
-		if (!strcmp(ptr->tag, "x_origin")) xorig = parse_math(ptr->parm, NULL, width);
-		else if (!strcmp(ptr->tag, "y_origin")) yorig = parse_math(ptr->parm, NULL, height);
+		if (!strcmp(ptr->tag, "x_origin")) xorig = (int)parse_math(ptr->parm, NULL, width);
+		else if (!strcmp(ptr->tag, "y_origin")) yorig = (int)parse_math(ptr->parm, NULL, height);
 		else if (!strcmp(ptr->tag, "affected_hue"))
 		{
 			if( isdigit( (int)ptr->parm[0] ) )
@@ -2159,10 +2203,10 @@ handle_asxml_tag_pad( ASImageXMLState *state, xml_elem_t* doc, xml_elem_t* parm,
 	int left = 0, top = 0, right = 0, bottom = 0;
 	LOCAL_DEBUG_OUT("doc = %p, parm = %p, imtmp = %p, width = %d, height = %d", doc, parm, imtmp, width, height ); 
 	for (ptr = parm ; ptr ; ptr = ptr->next) {
-		if (!strcmp(ptr->tag, "left"))   left = parse_math(ptr->parm, NULL, width);
-		else if (!strcmp(ptr->tag, "top"))    top = parse_math(ptr->parm, NULL, height);
-		else if (!strcmp(ptr->tag, "right"))  right = parse_math(ptr->parm, NULL, width);
-		else if (!strcmp(ptr->tag, "bottom")) bottom = parse_math(ptr->parm, NULL, height);
+		if (!strcmp(ptr->tag, "left"))   left = (int)parse_math(ptr->parm, NULL, width);
+		else if (!strcmp(ptr->tag, "top"))    top = (int)parse_math(ptr->parm, NULL, height);
+		else if (!strcmp(ptr->tag, "right"))  right = (int)parse_math(ptr->parm, NULL, width);
+		else if (!strcmp(ptr->tag, "bottom")) bottom = (int)parse_math(ptr->parm, NULL, height);
 		else if (!strcmp(ptr->tag, "color"))  parse_argb_color(ptr->parm, &color);
 	}
 	if( state->verbose > 1 )
