@@ -23,7 +23,7 @@
 #endif
 
 #define LOCAL_DEBUG
-/*#define DEBUG_SL2XIMAGE */
+#undef DEBUG_SL2XIMAGE
 #include <string.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -73,15 +73,15 @@ asvisual_empty_XErrorHandler (Display * dpy, XErrorEvent * event)
 /***************************************************************************/
 #if defined(LOCAL_DEBUG) && !defined(NO_DEBUG_OUTPUT)
 Status
-debug_AllocColor( const char *file, const char *func, int line, ASVisual *asv, Colormap cmap, XColor *pxcol )
+debug_AllocColor( const char *file, int line, ASVisual *asv, Colormap cmap, XColor *pxcol )
 {
 	Status sret ;
 	sret = XAllocColor( asv->dpy, cmap, pxcol );
-	show_progress( " XAllocColor in %s:%s():%d has %s -> cmap = %lX, pixel = %lu(%8.8lX), color = 0x%4.4X, 0x%4.4X, 0x%4.4X",
-				   file, func, line, (sret==0)?"failed":"succeeded", (long)cmap, (unsigned long)(pxcol->pixel), (unsigned long)(pxcol->pixel), pxcol->red, pxcol->green, pxcol->blue );
+	show_progress( " XAllocColor in %s:%d has %s -> cmap = %lX, pixel = %lu(%8.8lX), color = 0x%4.4X, 0x%4.4X, 0x%4.4X",
+				   file, line, (sret==0)?"failed":"succeeded", (long)cmap, (unsigned long)(pxcol->pixel), (unsigned long)(pxcol->pixel), pxcol->red, pxcol->green, pxcol->blue );
 	return sret;
 }
-#define ASV_ALLOC_COLOR(asv,cmap,pxcol)  debug_AllocColor(__FILE__, __FUNCTION__, __LINE__, (asv),(cmap),(pxcol))
+#define ASV_ALLOC_COLOR(asv,cmap,pxcol)  debug_AllocColor(__FILE__, __LINE__, (asv),(cmap),(pxcol))
 #else
 #define ASV_ALLOC_COLOR(asv,cmap,pxcol)  XAllocColor((asv)->dpy,(cmap),(pxcol))
 #endif
@@ -150,9 +150,9 @@ prepare_scanline( unsigned int width, unsigned int shift, ASScanline *reusable_m
 	/* we want to align data by 8 byte boundary (double)
 	 * to allow for code with less ifs and easier MMX/3Dnow utilization :*/
 	aligned_width = width + (width&0x00000001);
-	sl->buffer = ptr = safemalloc (((aligned_width*4)+16)*sizeof(CARD32));
+	sl->buffer = ptr = safemalloc (((aligned_width*4)+16)*sizeof(CARD32)+8);
 
-	sl->xc1 = sl->red 	= (CARD32*)(((long)ptr>>3)*8);
+	sl->xc1 = sl->red 	= (CARD32*)((((long)ptr+7)>>3)*8);
 	sl->xc2 = sl->green = sl->red   + aligned_width;
 	sl->xc3 = sl->blue 	= sl->green + aligned_width;
 	sl->alpha 	= sl->blue  + aligned_width;
@@ -1870,28 +1870,36 @@ void ximage2scanline32(ASVisual *asv, XImage *xim, ASScanline *sl, int y,  regis
 {
 	register CARD32 *r = sl->xc1+sl->offset_x, *g = sl->xc2+sl->offset_x, *b = sl->xc3+sl->offset_x;
 	register CARD32 *a = sl->alpha+sl->offset_x;
-	int max_i = MIN((unsigned int)(xim->width),sl->width-sl->offset_x);
+	int i = MIN((unsigned int)(xim->width),sl->width-sl->offset_x);
 	register CARD32 *src = (CARD32*)xim_data ;
-	register int i = 0;
+/*	src += sl->offset_x; */
+/*fprintf( stderr, "%d: ", y);*/
+
+#ifdef WORDS_BIGENDIAN
+	if( !asv->msb_first )
+#else
 	if( asv->msb_first )
+#endif
 	{
-		do
+		while (--i >= 0)
 		{
-			r[i] = (src[i]>>8)&0x0ff;
-			g[i] = (src[i]>>16)&0x0ff;
 			b[i] = (src[i]>>24)&0x0ff;
+			g[i] = (src[i]>>16)&0x0ff;
+			r[i] = (src[i]>>8)&0x0ff;
 			a[i] = src[i]&0x0ff;
-		}while(++i < max_i);
+/*			fprintf( stderr, "[%d->%8.8X %8.8X %8.8X %8.8X = %8.8X]", i, r[i], g[i], b[i], a[i], src[i]);*/
+		}
 	}else
 	{
-		do
+		while (--i >= 0)
 		{
+			a[i] = (src[i]>>24)&0x0ff;
 			r[i] = (src[i]>>16)&0x0ff;
 			g[i] = (src[i]>>8)&0x0ff;
 			b[i] =  src[i]&0x0ff;
-			a[i] = (src[i]>>24)&0x0ff;
-		}while(++i < max_i);
+		}
 	}
+/*fprintf( stderr, "\n");*/
 }
 
 void ximage2scanline16( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data )
@@ -2058,39 +2066,28 @@ void scanline2ximage32( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  regi
 	register CARD32 *r = sl->xc1+sl->offset_x, *g = sl->xc2+sl->offset_x, *b = sl->xc3+sl->offset_x;
 	register CARD32 *a = sl->alpha+sl->offset_x;
 	register int i = MIN((unsigned int)(xim->width),sl->width-sl->offset_x);
-	register CARD8 *src = (CARD8*)(xim_data+(i-1)*4) ;
+	register CARD32 *src = (CARD32*)xim_data;
+/*	src += sl->offset_x ; */
+/*fprintf( stderr, "%d: ", y);*/
+#ifdef WORDS_BIGENDIAN
+	if( !asv->msb_first )
+#else
 	if( asv->msb_first )
-		do
-		{
-			--i ;
-			{
-				src[0] = a[i];
-				src[1] = r[i];
- 				src[2] = g[i];
-				src[3] = b[i];
-				src -= 4 ;
-			}
-		}while(i);
+#endif
+		while( --i >= 0)
+		{ 
+			src[i] = (b[i]<<24)|(g[i]<<16)|(r[i]<<8)|a[i];
+/*			fprintf( stderr, "[%d->%8.8X %8.8X %8.8X %8.8X = %8.8X]", i, r[i], g[i], b[i], a[i], src[i]);  */
+		}
 	else
-		do
-		{
-			--i ;
-			{
-				src[3] = a[i];
-				src[2] = r[i];
-				src[1] = g[i];
-				src[0] = b[i];
-				src -= 4 ;
-			}
-		}while(i);
+		while( --i >= 0) src[i] = (a[i]<<24)|(r[i]<<16)|(g[i]<<8)|b[i];
+/*fprintf( stderr, "\n");*/
 #ifdef DEBUG_SL2XIMAGE
 	i = MIN((unsigned int)(xim->width),sl->width-sl->offset_x);
-	src = (CARD8*)(xim_data+(i-1)*4);
-	printf( "xim->width = %d, sl->width = %d, sl->offset = %d\n", xim->width, sl->width, sl->offset_x );
-	while(--i>=0 )
-	{
-		printf( "%2.2X.%2.2X.%2.2X.%2.2X ", src[0], src[1], src[2], src[3] );
-	}
+	src = (CARD32*)xim_data;
+	src += sl->offset_x;
+	printf( "%d: xim->width = %d, sl->width = %d, sl->offset = %d: ", y, xim->width, sl->width, sl->offset_x );
+	while(--i>=0 )	printf( "%8.8lX ", src[i] );
 	printf( "\n" );
 #endif
 }
