@@ -615,7 +615,7 @@ InitMyApp (  const char *app_class, int argc, char **argv, void (*version_func)
 
     set_output_threshold( as_app_args.verbosity_level );
     if(as_app_args.log_file)
-		if( freopen( as_app_args.log_file, get_flags( as_app_args.flags, ASS_Restarting)?"a":"w", stderr ) == NULL )
+		if( freopen( as_app_args.log_file, /*get_flags( as_app_args.flags, ASS_Restarting)?*/"a"/*:"w"*/, stderr ) == NULL )
 	    	show_system_error( "failed to redirect output into file \"%s\"", as_app_args.log_file );
 
 
@@ -671,7 +671,9 @@ free_func_hash ()
 /*********** end command line parsing **************************/
 static char *_as_known_terms[]	= 
 {	
+	"urxvt",
 	"aterm",
+	"x-terminal-emulator",
 	"rxvt  -tr -fg yellow -bg black",
 	"eterm -tr -tint blue -fg yellow -bg black",
 	"xterm -fg yellow -bg blue",
@@ -689,6 +691,7 @@ static char *_as_known_browsers[]	=
 };
 static char *_as_known_editors[]	= 
 {
+	"editor",
 	"$EDITOR",
 	"nedit",
 	"xemacs",
@@ -719,13 +722,15 @@ char *as_get_default_tool(ASToolType type)
 	for( i = 0 ; _as_known_tools[type][i] ; ++i ) 
 	{
 		char *tmp = _as_known_tools[type][i] ;
+		char *fullname = NULL ;
+		int res ;
 		if( tmp[0] == '$' ) 
 			tmp = copy_replace_envvar( tmp );
-	 	else
-			tmp = mystrdup(tmp);				   
-		if( is_executable_in_path (tmp) )
-			return tmp;
-		free( tmp );
+		res = get_executable_in_path (tmp, &fullname);
+		if (tmp != _as_known_tools[type][i])
+			free( tmp );
+		if( res > 0 )
+			return fullname;
 	}
 	return NULL;
 }
@@ -739,19 +744,17 @@ set_environment_tool_from_list( ASEnvironment *e, ASToolType type, char ** list,
 		if( list[i] )
 		{
 			char *tmp = list[i] ;
+			char *fullname = NULL ;
 			if( tmp[0] == '$' ) 
 				tmp = copy_replace_envvar( tmp );
-	 		else
-				tmp = mystrdup(tmp);				   
-			if( is_executable_in_path( tmp ) ) 
-			{	
-				e->tool_command[type] = tmp;
+			if( get_executable_in_path( tmp, &fullname ) ) 
+			{
+				e->tool_command[type] = fullname;
 				break;
 			}else
-			{
 				show_warning( "%s command %s is not in the path", _as_tools_name[type], tmp );
+			if (tmp != list[i])
 				free(tmp);	  
-			}
 		}	 
 	if( e->tool_command[type] == NULL ) 
 		e->tool_command[type] = as_get_default_tool(type);
@@ -841,8 +844,8 @@ destroy_asenvironment( ASEnvironment **penv )
 			int i ;
 			if( e->module_path )
 				free( e->module_path );
-			if( e->audio_path )
-				free( e->audio_path );
+			if( e->sound_path )
+				free( e->sound_path );
 			if( e->icon_path )
 				free( e->icon_path );
 			if( e->pixmap_path )
@@ -1079,7 +1082,7 @@ check_singleton_child (int singleton_id, Bool kill_it_to_death)
 }
 
 int
-spawn_child( const char *cmd, int singleton_id, int screen, Window w, int context, Bool do_fork, Bool pass_args, ... )
+spawn_child( const char *cmd, int singleton_id, int screen, const char *orig_display, Window w, int context, Bool do_fork, Bool pass_args, ... )
 {
     int pid = 0;
 
@@ -1163,8 +1166,8 @@ spawn_child( const char *cmd, int singleton_id, int screen, Window w, int contex
 			for( env_s = 0  ; envvars[env_s] != NULL ; ++env_s )
 				envp[env_s] = envvars[env_s] ;	
         
-		envp[env_s] = safemalloc(8+strlen(display)+1);
-		sprintf( envp[env_s], "DISPLAY=%s", display );
+		envp[env_s] = safemalloc(8+strlen(orig_display?orig_display:display)+1);
+		sprintf( envp[env_s], "DISPLAY=%s", orig_display?orig_display:display );
 		++env_s ;
 		if( Environment ) 
 		{
@@ -1197,7 +1200,11 @@ spawn_child( const char *cmd, int singleton_id, int screen, Window w, int contex
             while( i > 0 && isdigit(display[--i]) );
             if( display[i] == '.' )
                 display[i+1] = '\0' ;
-
+/*
+            This bit of code seems to break AS restarting
+            on Fedora 8. causing DISPLAY=":0.0" to
+            become DISPLAY=":0.".  -- Jeremy
+*/
             if( screen >= 0 )
                 screen_str = string_from_int( screen );
             if( w != None )
@@ -1205,7 +1212,7 @@ spawn_child( const char *cmd, int singleton_id, int screen, Window w, int contex
             if( context != C_NO_CONTEXT )
                 context_str = string_from_int( context );
 
-            len += 1+2+1+strlen( display );
+            len += 1+2+1+strlen( orig_display?orig_display:display );
             if( screen_str )
                 len += strlen(screen_str);
             len += 3 ;                         /* for "-s " */
@@ -1247,7 +1254,10 @@ spawn_child( const char *cmd, int singleton_id, int screen, Window w, int contex
         while(*ptr) ptr++;
         if( pass_args )
         {
-            ptr += sprintf( ptr, " -d %s%s -s", display, screen_str?screen_str:"" );
+			if (orig_display)
+    	        ptr += sprintf( ptr, " -d %s -s", orig_display );
+			else
+	            ptr += sprintf( ptr, " -d %s%s -s", display, screen_str?screen_str:"" );
             if ( get_flags( as_app_args.flags, ASS_Debugging) )
             {
                 strcpy( ptr, " --debug");
@@ -1266,7 +1276,7 @@ spawn_child( const char *cmd, int singleton_id, int screen, Window w, int contex
                 ptr += sprintf( ptr, " -g %s", as_app_args.override_share );
             if( as_app_args.verbosity_level != OUTPUT_DEFAULT_THRESHOLD )
                 ptr += sprintf( ptr, " -V %d", as_app_args.verbosity_level );
-LOCAL_DEBUG_OUT( "len = %d, cmdl = \"%s\" strlen = %d, locale = \"%s\", ptr-cmdl = %d", len, cmdl, strlen(cmdl), as_app_args.locale, ptr-cmdl );
+LOCAL_DEBUG_OUT( "len = %d, cmdl = \"%s\" strlen = %d, locale = \"%s\", ptr-cmdl = %d", len, cmdl, (int)strlen(cmdl), as_app_args.locale, (int)(ptr-cmdl) );
 			if( as_app_args.locale && as_app_args.locale[0] && !isspace(as_app_args.locale[0]))
                 ptr += sprintf( ptr, " -L %s", as_app_args.locale );
 			
@@ -1286,7 +1296,7 @@ LOCAL_DEBUG_OUT( "len = %d, cmdl = \"%s\" strlen = %d, locale = \"%s\", ptr-cmdl
             *(ptr++) = ' ';
             strcpy( ptr, arg );
             while(*ptr) ptr++;
-LOCAL_DEBUG_OUT( "len = %d, cmdl = \"%s\" strlen = %d", len, cmdl, strlen(cmdl) );
+LOCAL_DEBUG_OUT( "len = %d, cmdl = \"%s\" strlen = %d", len, cmdl, (int)strlen(cmdl) );
         
 		}
         va_end(ap);
@@ -1298,7 +1308,7 @@ LOCAL_DEBUG_OUT( "len = %d, cmdl = \"%s\" strlen = %d", len, cmdl, strlen(cmdl)
         }
 		strcpy (ptr, do_fork?" &\n":"\n");
 
-LOCAL_DEBUG_OUT( "len = %d, cmdl = \"%s\" strlen = %d", len, cmdl, strlen(cmdl) );
+LOCAL_DEBUG_OUT( "len = %d, cmdl = \"%s\" strlen = %d", len, cmdl, (int)strlen(cmdl) );
 #if defined(LOCAL_DEBUG) && !defined(NO_DEBUG_OUTPUT)
 		{
 			FILE *fff = fopen ("/tmp/afterstep.exec.log", "a");
