@@ -13,6 +13,7 @@ struct icon_t;
 struct button_t;
 struct ASImageManager;
 struct ASBalloon;
+struct ASBalloonState;
 
 typedef struct ASTBtnData{
     struct ASImage *unpressed ;
@@ -156,6 +157,9 @@ typedef struct ASTBarData {
 
 #define BAR_FLAGS_VERTICAL      (0x01<<17)     /* vertical label */
 #define BAR_FLAGS_IMAGE_BACK    (0x01<<18)     /* back represents an icon instead of  */
+#define BAR_FLAGS_CROP_UNFOCUSED_BACK   (0x01<<19)     /* crop background image in relation to canvas origin  */
+#define BAR_FLAGS_CROP_FOCUSED_BACK     (0x01<<20)     /* crop background image in relation to canvas origin  */
+#define BAR_FLAGS_CROP_BACK  (BAR_FLAGS_CROP_FOCUSED_BACK|BAR_FLAGS_CROP_UNFOCUSED_BACK)
 
     ASFlagType  state ;
     unsigned long context ;
@@ -203,7 +207,7 @@ Bool set_astbar_size( ASTBarData *tbar, unsigned int width, unsigned int height
 Bool set_astbar_hilite( ASTBarData *tbar, unsigned int state, ASFlagType hilite );
 Bool set_astbar_composition_method( ASTBarData *tbar, unsigned int state, unsigned char method );
 Bool set_astbar_huesat( ASTBarData *tbar, unsigned int state, int hue, int sat );
-Bool set_astbar_style_ptr (ASTBarData * tbar, unsigned int state, struct MyStyle *style);
+Bool set_astbar_style_ptr (ASTBarData * tbar, int state, struct MyStyle *style);
 Bool set_astbar_style( ASTBarData *tbar, unsigned int state, const char *style_name );
 Bool set_astbar_flip( ASTBarData * tbar, int flip );
 Bool invalidate_astbar_style (ASTBarData * tbar, int state);
@@ -242,9 +246,11 @@ Bool  trace_render_astbar (ASTBarData * tbar, ASCanvas * pc, const char *file, i
 #else
 Bool render_astbar( ASTBarData *tbar, ASCanvas *pc );
 #endif
+Bool render_astbar_cached_back (ASTBarData * tbar, ASCanvas * pc, ASImage **cache, ASCanvas *origin_canvas);
 
 void on_astbar_pointer_action( ASTBarData *tbar, int context, Bool leave, Bool pointer_moved );
 void set_astbar_balloon( ASTBarData *tbar, int context, const char *text, unsigned long encoding );
+void set_astbar_balloon2( ASTBarData *tbar, struct ASBalloonState *balloon_state, int context, const char *text, unsigned long encoding );
 
 
 #ifdef __cplusplus
