@@ -116,22 +116,30 @@ typedef struct ConfigItem
 
 typedef struct flag_options_xref
 {
-  unsigned long flag;
-  int id_on, id_off;
+  	unsigned long flag;
+  	int id_on, id_off;
+  
+  	ptrdiff_t	flag_field_offset ; 
+	ptrdiff_t	set_flag_field_offset ; 
 }
 flag_options_xref;
 
+
 /* this functions return 1 on success - 0 otherwise */
 int ReadConfigItem (ConfigItem * item, FreeStorageElem * stored);
+Bool ReadConfigItemToStruct( void *struct_ptr, ptrdiff_t set_flags_offset, FreeStorageElem * stored );
+Bool ReadCompositeFlagsConfigItem( void *struct_ptr, ptrdiff_t flags_field_offset, FreeStorageElem * stored );
+
 /* indexed flags cannot be handled by ReadFlagItem - it will return 0
    for those - handle them manually using ReadConfigItem */
 int ReadFlagItem (unsigned long *set_flags, unsigned long *flags,
 		  FreeStorageElem * stored, flag_options_xref * xref);
+int ReadFlagItemAuto (void *config_struct, ptrdiff_t default_set_flags_offset, FreeStorageElem * stored, flag_options_xref * xref);
 
 /* really defined in libAfterBase/mystring.h (unsigned long*)*/
 void set_string (char **target, char *string);
 #define set_string_value(ptarget,string,pset_flags,flag) \
-	do {set_string(ptarget,string); if(pset_flags)set_flags(*(pset_flags),(flag));}while(0)
+	do {ASFlagType *_fp_ = (pset_flags);set_string(ptarget,string); if(_fp_)set_flags(*_fp_,(flag));}while(0)
 #define set_scalar_value(ptarget,val,pset_flags,flag) \
 	do {*(ptarget)=(val); if(pset_flags)set_flags(*(pset_flags),(flag));}while(0)
 #define set_size_geometry(pw,ph,pitem,pset_flags,flag) \
