@@ -335,7 +335,11 @@ NewConfig (char *myname, SyntaxDef * syntax, ConfigDataType type, ConfigData  so
 		}
 
 	if (new_conf->fd != -1 && new_conf->fp == NULL)
+	{
         new_conf->fp = fdopen (new_conf->fd, get_flags(new_conf->flags, CP_ReadLines)?"rt":"rb");
+        set_flags( new_conf->flags, CP_NeedToFCloseFile);
+	}
+		
 
 	new_conf->myname = mystrdup(myname);
 	new_conf->current_syntax = NULL;
@@ -455,6 +459,8 @@ DestroyConfig (ConfigDef * config)
 		FreeSyntaxHash (config->syntax);
     if (get_flags(config->flags, CP_NeedToCloseFile) && config->fd != -1)
 		close (config->fd);
+    if (get_flags(config->flags, CP_NeedToFCloseFile) && config->fp != NULL)
+		fclose (config->fp);
 	free (config);
 }
 
@@ -538,7 +544,7 @@ GetToNextLine (ConfigDef * config)
             if( get_flags( config->flags, CP_ReadLines ) )
             {
 LOCAL_DEBUG_OUT( "Reading Lines ...%s", "" );
-                if (!fgets (config->buffer, config->buffer_size, config->fp))
+                if (!fgets (config->buffer, config->buffer_size-1, config->fp))
                     return NULL;
                 config->bytes_in = strlen(config->buffer);
                 config->cursor = &(config->buffer[0]);
@@ -548,7 +554,7 @@ LOCAL_DEBUG_OUT( "Reading Lines ...%s", "" );
                 register char *ptr = config->buffer ;
 
 LOCAL_DEBUG_OUT( "Reading Buffer ...%s", "" );
-                config->bytes_in = read (config->fd, ptr, config->buffer_size);
+                config->bytes_in = read (config->fd, ptr, config->buffer_size-1);
                 if (config->bytes_in <= 0)
                     return NULL;
                 print_trimmed_str( "new data begins with", ptr );
