@@ -64,6 +64,8 @@ init_aswindow_list()
 
     list->circulate_list = create_asvector( sizeof(ASWindow*) );
     list->sticky_list = create_asvector( sizeof(ASWindow*) );
+	
+	list->stacking_order = create_asvector( sizeof(ASWindow*) );
 
     Scr.on_dead_window = on_dead_aswindow ;
 
@@ -273,6 +275,10 @@ make_aswindow_cmd_iter_func(void *data, void *aux_data)
     {
 		Bool same_host = (asw->hints->client_host == NULL || mystrcasecmp( asw->hints->client_host, swad->this_host )== 0);
 
+		/* don't want to save windows with short life span - wharf subfolders, menus, dialogs, etc. */
+		if( ASWIN_HFLAGS(asw,AS_ShortLived) ) 
+			return True;
+
 		/* don't want to save modules - those are started from autoexec anyways */
 		if( ASWIN_HFLAGS(asw,AS_Module) ) 
 		{	
@@ -473,63 +479,13 @@ destroy_aswindow_list( ASWindowList **list, Bool restore_root )
 			destroy_ashash(&((*list)->bookmarks));
             destroy_asvector(&((*list)->sticky_list));
             destroy_asvector(&((*list)->circulate_list));
+			destroy_asvector(&((*list)->stacking_order));
+			
             free(*list);
             *list = NULL ;
         }
 }
 
-void
-publish_aswindow_list( ASWindowList *list, Bool stacking_only )
-{
-	int clients_num = list->clients->count, used = 0 ;
-	Window *client_list	;
-	int i ;
-    ASLayer **layers = NULL ;
-    unsigned long layers_in ;
-	
-	if( clients_num == 0 ) 
-		return;
-
-	client_list = safecalloc( clients_num, sizeof(Window));
-	if( !stacking_only ) 
-	{
-		ASBiDirElem *curr = LIST_START(list->clients);
-		used = 0 ;
-		while( curr && used < clients_num )
-		{
-			ASWindow *asw = (ASWindow*)LISTELEM_DATA(curr);
-			client_list[used++] = asw->w ;
-			LIST_GOTO_NEXT(curr);	
-		}	 
-		LOCAL_DEBUG_OUT( "Setting Client List property to include %d windows (clients_num = %d) ", used, clients_num );
-		set_clients_list (Scr.wmprops, client_list, used);
-	}		  
-    
-    layers = safecalloc( list->layers->items_num, sizeof(ASLayer*) );
-	if( (layers_in = sort_hash_items (list->layers, NULL, (void**)layers, 0)) == 0 )
-        return ;
-
-    used = clients_num ;       /* we have to reverse the order - bottommost window comes first */
-	for( i = 0 ; i < layers_in ; i++ )
-    {
-        register int k, end_k = PVECTOR_USED(layers[i]->members) ;
-        ASWindow **members = PVECTOR_HEAD(ASWindow*,layers[i]->members);
-        if( end_k > used )
-            end_k = used ;
-        for( k = 0 ; k < end_k ; k++ )
-        {
-            register ASWindow *asw = members[k] ;
-            if( !ASWIN_GET_FLAGS(asw, AS_Dead) )
-            	client_list[--used] = asw->w;
-        }
-    }
-	set_stacking_order (Scr.wmprops, &client_list[used], clients_num - used);
-
-	free( layers );
-	free( client_list );
-		   
-}
-
 /*************************************************************************
  * We maintain crossreference of X Window ID to ASWindow structure - that is
  * faster then using XContext since we don't have to worry about multiprocessing,
@@ -808,6 +764,49 @@ destroy_aslayer  (ASHashableValue value, void *data)
 /********************************************************************************/
 /* ASWindow management */
 
+ASWindow*
+find_group_lead_aswindow( Window id )
+{
+	ASWindow *gl = window2ASWindow( id );
+	if( gl == NULL ) 
+	{/* let's find previous window with the same group lead */
+		ASBiDirElem *curr = LIST_START(Scr.Windows->clients);
+		while( curr )
+		{
+			ASWindow *asw = (ASWindow*)LISTELEM_DATA(curr);
+			if( asw->hints->group_lead == id ) 
+			{
+				gl = asw ; 
+				break;
+			}
+			LIST_GOTO_NEXT(curr);	
+		}	 
+	}
+	if( gl )
+	{
+		while( gl->group_lead != NULL ) 
+		{
+			if( ASWIN_GET_FLAGS(gl->group_lead, AS_Dead) )
+			{
+    		    if( gl->group_lead->group_members )
+	        	    vector_remove_elem( gl->group_lead->group_members, &gl );
+				gl->group_lead = NULL ; 
+			}else
+				gl = gl->group_lead;
+		}
+	}
+	return gl;
+}
+
+static void
+add_member_to_group_lead( ASWindow *group_lead, ASWindow *member ) 
+{
+    member->group_lead = group_lead ;
+    if( group_lead->group_members == NULL )
+        group_lead->group_members = create_asvector( sizeof( ASWindow* ) );
+    vector_insert_elem( group_lead->group_members, &member, 1, NULL, True );
+}
+
 void
 tie_aswindow( ASWindow *t )
 {
@@ -815,7 +814,20 @@ tie_aswindow( ASWindow *t )
     {
         ASWindow *transient_owner  = window2ASWindow(t->hints->transient_for);
         if( transient_owner != NULL )
-        {
+        {/* we want to find the topmost window */
+   			while( transient_owner->transient_owner != NULL ) 
+			{
+				if( ASWIN_GET_FLAGS(transient_owner->transient_owner, AS_Dead) )
+				{
+					ASWindow *t = transient_owner ;
+			        if( t->transient_owner->transients != NULL )
+            			vector_remove_elem( t->transient_owner->transients, &t );
+					t->transient_owner = NULL ; 
+					add_aswindow_to_layer( t, ASWIN_LAYER(t) );				
+				}else
+					transient_owner = transient_owner->transient_owner ;
+			}
+				
             t->transient_owner = transient_owner ;
             if( transient_owner->transients == NULL )
                 transient_owner->transients = create_asvector( sizeof( ASWindow* ) );
@@ -824,38 +836,60 @@ tie_aswindow( ASWindow *t )
     }
     if( t->hints->group_lead != None )
     {
-        ASWindow *group_lead  = window2ASWindow( t->hints->group_lead );
-        if( group_lead != NULL )
-        {
-            t->group_lead = group_lead ;
-            if( group_lead->group_members == NULL )
-                group_lead->group_members = create_asvector( sizeof( ASWindow* ) );
-            vector_insert_elem( group_lead->group_members, &t, 1, NULL, True );
-        }
+        ASWindow *group_lead  = find_group_lead_aswindow( t->hints->group_lead );
+        if( group_lead != NULL && group_lead != t )
+			add_member_to_group_lead( group_lead, t );
     }
 }
 
 void
 untie_aswindow( ASWindow *t )
 {
+	ASWindow **sublist; 
+	int i ; 
+
     if( t->transient_owner != NULL && t->transient_owner->magic == MAGIC_ASWINDOW )
     {
         if( t->transient_owner != NULL )
             vector_remove_elem( t->transient_owner->transients, &t );
         t->transient_owner = NULL ;
     }
+	if( t->transients && PVECTOR_USED(t->transients) > 0 )
+	{
+		sublist = PVECTOR_HEAD(ASWindow*,t->transients);
+		for( i = 0 ; i < PVECTOR_USED(t->transients) ; ++i ) 
+			if( sublist[i] && sublist[i]->magic == MAGIC_ASWINDOW && sublist[i]->transient_owner == t ) 
+			{	/* we may need to delete this windows actually */
+				sublist[i]->transient_owner = NULL ;
+				add_aswindow_to_layer( sublist[i], ASWIN_LAYER(sublist[i]) );				
+			}
+	}
     if( t->group_lead && t->group_lead->magic == MAGIC_ASWINDOW )
     {
         if( t->group_lead->group_members )
             vector_remove_elem( t->group_lead->group_members, &t );
         t->group_lead = NULL ;
     }
+	if( t->group_members && PVECTOR_USED(t->group_members) > 0 )
+	{
+		ASWindow *new_gl ;
+		sublist = PVECTOR_HEAD(ASWindow*,t->group_members);
+		new_gl = sublist[0] ; 
+		new_gl->group_lead = NULL ; 
+		
+		for( i = 1 ; i < PVECTOR_USED(t->group_members) ; ++i ) 
+			if( sublist[i] && sublist[i]->magic == MAGIC_ASWINDOW && sublist[i]->group_lead == t ) 
+			{
+				sublist[i]->group_lead = NULL ;
+				add_member_to_group_lead( new_gl, sublist[i] ); 				
+			}
+	}
 }
 
 void
 add_aswindow_to_layer( ASWindow *asw, int layer )
 {
-    if( !AS_ASSERT(asw) )
+    if( !AS_ASSERT(asw) && asw->transient_owner == NULL )
     {
         ASLayer  *dst_layer = get_aslayer( layer, Scr.Windows );
         /* inserting window into the top of the new layer */
@@ -871,16 +905,16 @@ remove_aswindow_from_layer( ASWindow *asw, int layer )
     if( !AS_ASSERT(asw) )
     {
         ASLayer  *src_layer = get_aslayer( layer, Scr.Windows );
-    LOCAL_DEBUG_OUT( "removing window %p from layer %p (%d)", asw, src_layer, layer );
+	    LOCAL_DEBUG_OUT( "removing window %p from layer %p (%d)", asw, src_layer, layer );
         if( !AS_ASSERT(src_layer) )
-	{
-LOCAL_DEBUG_OUT( "can be found at index %d", vector_find_data(	src_layer->members, &asw ) );
-	    while( vector_find_data( src_layer->members, &asw ) < src_layer->members->used )
-	    {
+		{
+			LOCAL_DEBUG_OUT( "can be found at index %d", vector_find_data(	src_layer->members, &asw ) );
+		    while( vector_find_data( src_layer->members, &asw ) < src_layer->members->used )
+	    	{
     	        vector_remove_elem( src_layer->members, &asw );
-        LOCAL_DEBUG_OUT( "after delition can be found at index %d(used%d)", vector_find_data(   src_layer->members, &asw ), src_layer->members->used );
-	    }
-	}
+	    	    LOCAL_DEBUG_OUT( "after deletion can be found at index %d(used%d)", vector_find_data(   src_layer->members, &asw ), src_layer->members->used );
+		    }
+		}
     }
 }
 
@@ -894,9 +928,10 @@ enlist_aswindow( ASWindow *t )
     append_bidirelem( Scr.Windows->clients, t );
     vector_insert_elem( Scr.Windows->circulate_list, &t, 1, NULL, True );
 
-    add_aswindow_to_layer( t, ASWIN_LAYER(t) );
     tie_aswindow( t );
-	
+	if( t->transient_owner == NULL )
+    	add_aswindow_to_layer( t, ASWIN_LAYER(t) );
+		
 	publish_aswindow_list( Scr.Windows, False );	
     
 	return True;
@@ -905,7 +940,6 @@ enlist_aswindow( ASWindow *t )
 void
 delist_aswindow( ASWindow *t )
 {
-    Bool skip_winlist;
     if( Scr.Windows == NULL )
         return ;
 
@@ -919,11 +953,11 @@ delist_aswindow( ASWindow *t )
         vector_remove_elem( Scr.Windows->sticky_list, &t );
 
     remove_aswindow_from_layer( t, ASWIN_LAYER(t) );
-
-    untie_aswindow( t );
-    skip_winlist = get_flags(t->hints->flags, AS_SkipWinList);
+	
+	vector_remove_elem( Scr.Windows->stacking_order, &t );
+    
+	untie_aswindow( t );
     discard_bidirelem( Scr.Windows->clients, t );
-
 	publish_aswindow_list( Scr.Windows, False );	   
 }
 
@@ -1038,15 +1072,72 @@ update_visibility( int desk )
 }
 #endif
 
+#if 0
+/********************************************************************************************/
+/* old version of code - stacking window IDs instead of pointers to ASWindow */
+static void 
+stack_transients( ASWindow *asw, ASVector *ids, Bool use_frame_ids ) 
+{
+	int tnum = PVECTOR_USED(asw->transients);
+    LOCAL_DEBUG_OUT( "Client %lX has %d transients", asw->w, tnum );
+	if( tnum > 0 )
+	{/* need to collect all the transients and stick them in front of us in order of creation */
+		ASWindow **sublist = PVECTOR_HEAD(ASWindow*,asw->transients);
+		int curr ;
+		for( curr = 0 ; curr < tnum ; ++curr )
+	        if( !ASWIN_GET_FLAGS(sublist[curr], AS_Dead) )
+			{
+				Window w = use_frame_ids?get_window_frame(sublist[curr]):sublist[curr]->w;
+				if( vector_find_data(ids, &w)  >= PVECTOR_USED(ids) )
+				{
+			    	LOCAL_DEBUG_OUT( "Adding transient%s #%d - %lX", use_frame_ids?"'s frame":"", curr, w );
+					vector_insert_elem(ids, &w, 1, NULL, False);
+				}
+			}
+	}
+}
+
+static void
+stack_layer_windows( ASLayer *layer, ASVector *ids, int desk, Bool use_frame_ids ) 
+{
+    int k;
+    ASWindow **members = PVECTOR_HEAD(ASWindow*,layer->members);
+	
+    LOCAL_DEBUG_OUT( "layer %d, end_k = %d", layer->layer, PVECTOR_USED(layer->members) );
+    for( k = 0 ; k < PVECTOR_USED(layer->members) ; k++ )
+    {
+        register ASWindow *asw = members[k] ;
+        if( ASWIN_DESK(asw) == desk && !ASWIN_GET_FLAGS(asw, AS_Dead) )
+		{
+			Window w  = use_frame_ids?get_window_frame(asw):asw->w; 
+		    LOCAL_DEBUG_OUT( "Group Lead %p", asw->group_lead );
+			if( asw->group_lead ) 
+			{/* transients for any window in the group go on top of non-transients in the group */
+				ASWindow **sublist = PVECTOR_HEAD(ASWindow*,asw->group_lead->group_members);
+				int curr = PVECTOR_USED(asw->group_lead->group_members);
+			    LOCAL_DEBUG_OUT( "Group members %d", curr );
+				if( asw->group_lead->transients && !ASWIN_GET_FLAGS(asw->group_lead, AS_Dead) )
+					stack_transients( asw->group_lead, ids, use_frame_ids );
+				while( --curr >= 0 ) 
+				{/* most recent group member should have their transients above others */
+					if( !ASWIN_GET_FLAGS(sublist[curr], AS_Dead) && sublist[curr]->transients ) 
+						stack_transients( sublist[curr], ids, use_frame_ids );
+				}
+			}else if( asw->transients )
+				stack_transients( asw, ids, use_frame_ids );
+
+		    LOCAL_DEBUG_OUT( "Adding client%s - %lX", use_frame_ids?"'s frame":"", w );
+			vector_insert_elem(ids, &w, 1, NULL, False);
+		}
+    }
+}
+
 void
 restack_window_list( int desk, Bool send_msg_only )
 {
     static ASVector *ids = NULL ;
-    static ASVector *layers = NULL ;
     unsigned long layers_in, i ;
-    register long windows_num ;
     ASLayer **l ;
-    Window  *windows ;
 
     if( !IsValidDesk(desk) )
     {
@@ -1060,75 +1151,47 @@ restack_window_list( int desk, Bool send_msg_only )
     if( Scr.Windows->clients->count == 0)
         return;
 
-    if( layers == NULL )
-        layers = create_asvector( sizeof(ASLayer*) );
-    if( Scr.Windows->layers->items_num > layers->allocated )
-        realloc_vector( layers, Scr.Windows->layers->items_num );
+    if( (layers_in = get_sorted_layers_vector( &layers )) == 0 )
+        return ;
 
     if( ids == NULL )
         ids = create_asvector( sizeof(Window) );
     else
         flush_vector( ids );
-    if( Scr.Windows->clients->count+1 > ids->allocated )
-        realloc_vector( ids, Scr.Windows->clients->count+1 );
-
-    if( (layers_in = sort_hash_items (Scr.Windows->layers, NULL, (void**)VECTOR_HEAD_RAW(*layers), 0)) == 0 )
-        return ;
+    if( Scr.Windows->clients->count+2 > ids->allocated )
+        realloc_vector( ids, Scr.Windows->clients->count+2 );
 
     l = PVECTOR_HEAD(ASLayer*,layers);
-    windows = PVECTOR_HEAD(Window,ids);
-    windows_num = 0 ;
     LOCAL_DEBUG_OUT( "filling up array with window IDs: layers_in = %ld, Total clients = %d", layers_in, Scr.Windows->clients->count );
+	flush_vector( ids );
     for( i = 0 ; i < layers_in ; i++ )
-    {
-        register int k, end_k = PVECTOR_USED(l[i]->members) ;
-        ASWindow **members = PVECTOR_HEAD(ASWindow*,l[i]->members);
-        if( end_k > ids->allocated )
-            end_k = ids->allocated ;
-        LOCAL_DEBUG_OUT( "layer[%ld] = %d, end_k = %d", i, l[i]->layer, end_k );
-        for( k = 0 ; k < end_k ; k++ )
-        {
-            register ASWindow *asw = members[k] ;
-            if( ASWIN_DESK(asw) == desk )
-                if( !ASWIN_GET_FLAGS(asw, AS_Dead) )
-                    windows[windows_num++] = asw->w;
-        }
-    }
-    LOCAL_DEBUG_OUT( "Sending stacking order: windows_num = %ld, ", windows_num );
-    PVECTOR_USED(ids) = windows_num ;
-    SendStackingOrder (-1, M_STACKING_ORDER, desk, ids);
-	
-	publish_aswindow_list( Scr.Windows, True );
+		stack_layer_windows( l[i], ids, desk, False );
 
+    LOCAL_DEBUG_OUT( "Sending stacking order: windows_num = %d, ", PVECTOR_USED(ids) );
+    SendStackingOrder (-1, M_STACKING_ORDER, desk, ids);
+
+	publish_aswindow_list( Scr.Windows, True );
+	
     if( !send_msg_only )
     {
+		int windows_num ; 
+		Window cw = get_desktop_cover_window();
         l = PVECTOR_HEAD(ASLayer*,layers);
-        windows = PVECTOR_HEAD(Window,ids);
-        windows_num = 0 ;
-        if( get_desktop_cover_window() )
-        {    
-            windows[0] = get_desktop_cover_window();
-            ++windows_num ;
-        }
+
+		flush_vector( ids );
+        if( cw != None )
+			vector_insert_elem(ids, &cw, 1, NULL, True);
+	
         LOCAL_DEBUG_OUT( "filling up array with window frame IDs: layers_in = %ld, ", layers_in );
         for( i = 0 ; i < layers_in ; i++ )
-        {
-            register int k, end_k = PVECTOR_USED(l[i]->members) ;
-            register ASWindow **members = PVECTOR_HEAD(ASWindow*,l[i]->members);
-            if( windows_num+end_k > ids->allocated )
-                end_k = ids->allocated - windows_num ;
-            LOCAL_DEBUG_OUT( "layer[%ld] = %d, end_k = %d", i, l[i]->layer, end_k );
-            for( k = 0 ; k < end_k ; k++ )
-                if( ASWIN_DESK(members[k]) == desk && !ASWIN_GET_FLAGS(members[k], AS_Dead))
-				{
-                    windows[windows_num++] = get_window_frame(members[k]);
-					LOCAL_DEBUG_OUT( "\t windows[%d] = %lX", k, windows[windows_num-1] );
-				}
-        }
+			stack_layer_windows( l[i], ids, desk, True );
 
-        LOCAL_DEBUG_OUT( "Setting stacking order: windows_num = %ld, ", windows_num );
+		windows_num = PVECTOR_USED(ids);
+        LOCAL_DEBUG_OUT( "Setting stacking order: windows_num = %d, ", windows_num );
         if( windows_num > 0 )
         {
+		    Window  *windows = PVECTOR_HEAD(Window,ids);
+			
             XRaiseWindow( dpy, windows[0] );
             if( windows_num > 1 )
                 XRestackWindows( dpy, windows, windows_num );
@@ -1139,6 +1202,277 @@ restack_window_list( int desk, Bool send_msg_only )
     }
 }
 
+#endif
+/********************************************************************************************/
+/********************************************************************************************/
+/********************************************************************************************/
+static int
+get_sorted_layers_vector( ASVector **layers ) 
+{
+	if( *layers == NULL ) 
+        *layers = create_asvector( sizeof(ASLayer*) );
+    if( Scr.Windows->layers->items_num > (*layers)->allocated )
+        realloc_vector( *layers, Scr.Windows->layers->items_num );
+
+	return sort_hash_items (Scr.Windows->layers, NULL, (void**)VECTOR_HEAD_RAW(**layers), 0);	
+}
+
+static void 
+stack_transients( ASWindow *asw, ASVector *list) 
+{
+	int tnum = PVECTOR_USED(asw->transients);
+    LOCAL_DEBUG_OUT( "Client %lX has %d transients", asw->w, tnum );
+	if( tnum > 0 )
+	{/* need to collect all the transients and stick them in front of us in order of creation */
+		ASWindow **sublist = PVECTOR_HEAD(ASWindow*,asw->transients);
+		int curr ;
+		for( curr = 0 ; curr < tnum ; ++curr )
+	        if( !ASWIN_GET_FLAGS(sublist[curr], AS_Dead) )
+			{
+				if( vector_find_data(list, &sublist[curr])  >= PVECTOR_USED(list) )
+				{
+			    	LOCAL_DEBUG_OUT( "Adding transient #%d - %p, w = %lX, frame = %lX", curr, sublist[curr], sublist[curr]->w, sublist[curr]->frame );
+					vector_insert_elem(list, &sublist[curr], 1, NULL, False);
+				}
+			}
+	}
+}
+
+static inline void
+stack_layer_windows( ASLayer *layer, ASVector *list ) 
+{
+    int k;
+    ASWindow **members = PVECTOR_HEAD(ASWindow*,layer->members);
+	
+    LOCAL_DEBUG_OUT( "layer %d, end_k = %d", layer->layer, PVECTOR_USED(layer->members) );
+    for( k = 0 ; k < PVECTOR_USED(layer->members) ; k++ )
+    {
+        ASWindow *asw = members[k] ;
+        if( !ASWIN_GET_FLAGS(asw, AS_Dead) )
+		{
+		    LOCAL_DEBUG_OUT( "Group Lead %p", asw->group_lead );
+			if( asw->group_lead ) 
+			{/* transients for any window in the group go on top of non-transients in the group */
+				ASWindow **sublist = PVECTOR_HEAD(ASWindow*,asw->group_lead->group_members);
+				int curr = PVECTOR_USED(asw->group_lead->group_members);
+			    LOCAL_DEBUG_OUT( "Group members %d", curr );
+				if( asw->group_lead->transients && !ASWIN_GET_FLAGS(asw->group_lead, AS_Dead) )
+					stack_transients( asw->group_lead, list );
+				while( --curr >= 0 ) 
+				{/* most recent group member should have their transients above others */
+					if( !ASWIN_GET_FLAGS(sublist[curr], AS_Dead) && sublist[curr]->transients ) 
+						stack_transients( sublist[curr], list );
+				}
+			}else if( asw->transients )
+				stack_transients( asw, list );
+
+		    LOCAL_DEBUG_OUT( "Adding client - %p, w = %lX, frame = %lX", asw, asw->w, asw->frame );
+			vector_insert_elem(list, &asw, 1, NULL, False);
+		}
+    }
+}
+
+static ASVector *__as_scratch_layers = NULL; 
+
+void
+free_scratch_layers_vector()
+{
+	if( __as_scratch_layers ) 
+		destroy_asvector( &__as_scratch_layers );
+}
+
+ASVector *
+get_scratch_layers_vector()
+{
+    if( __as_scratch_layers == NULL )
+        __as_scratch_layers = create_asvector( sizeof(ASLayer*) );
+	else
+        flush_vector( __as_scratch_layers );
+
+	return __as_scratch_layers;	
+}
+
+
+void
+update_stacking_order()
+{
+   	ASVector *layers = get_scratch_layers_vector() ;
+    unsigned long layers_in, i ;
+    ASLayer **l ;
+	ASVector *list = Scr.Windows->stacking_order ; 
+
+	flush_vector( list );	
+    if( Scr.Windows->clients->count == 0)
+        return;
+
+    if( (layers_in = get_sorted_layers_vector( &layers )) == 0 )
+        return ;
+
+    realloc_vector( list, Scr.Windows->clients->count );
+    l = PVECTOR_HEAD(ASLayer*,layers);
+    for( i = 0 ; i < layers_in ; i++ )
+		stack_layer_windows( l[i], list );
+}
+
+static ASWindow ** 
+get_stacking_orger_list(ASWindowList *list, int *stack_len_return )
+{
+	int stack_len = PVECTOR_USED(list->stacking_order);
+	if( stack_len == 0 ) 
+	{
+		update_stacking_order();	
+		stack_len = PVECTOR_USED(list->stacking_order);
+	}
+	*stack_len_return = stack_len ;
+	return PVECTOR_HEAD(ASWindow*, Scr.Windows->stacking_order);
+}
+
+static ASVector *__as_scratch_ids = NULL; 
+
+void
+free_scratch_ids_vector()
+{
+	if( __as_scratch_ids ) 
+		destroy_asvector( &__as_scratch_ids );
+}
+
+ASVector *
+get_scratch_ids_vector()
+{
+    if( __as_scratch_ids == NULL )
+        __as_scratch_ids = create_asvector( sizeof(Window) );
+	else
+        flush_vector( __as_scratch_ids );
+	
+    if( Scr.Windows->clients->count+2 > __as_scratch_ids->allocated )
+        realloc_vector( __as_scratch_ids, Scr.Windows->clients->count+2 );
+	
+	return __as_scratch_ids;	
+}
+
+void 
+send_stacking_order( int desk )
+{
+    if( Scr.Windows->clients->count > 0)
+	{
+		int i ; 
+		int stack_len = 0;
+		ASWindow **stack = get_stacking_orger_list(Scr.Windows, &stack_len );
+		ASVector *ids = get_scratch_ids_vector();
+		
+		for( i = 0 ; i < stack_len ; ++i )
+			vector_insert_elem(ids, &(stack[i]->w), 1, NULL, False);
+
+	    SendStackingOrder (-1, M_STACKING_ORDER, desk, ids);
+	}
+}
+
+void 
+apply_stacking_order( int desk )
+{
+    if( Scr.Windows->clients->count > 0)
+	{
+		int i ; 
+		int stack_len = 0;
+		ASWindow **stack = get_stacking_orger_list(Scr.Windows, &stack_len );
+		ASVector *ids = get_scratch_ids_vector();
+		Window cw = get_desktop_cover_window();
+		int windows_num ; 
+		
+        if( cw != None )
+			vector_insert_elem(ids, &cw, 1, NULL, True);
+
+		for( i = 0 ; i < stack_len ; ++i )
+			vector_insert_elem(ids, &(stack[i]->frame), 1, NULL, False);
+
+		windows_num = PVECTOR_USED(ids);
+        LOCAL_DEBUG_OUT( "Setting stacking order: windows_num = %d, ", windows_num );
+        if( windows_num > 0 )
+        {
+		    Window  *windows = PVECTOR_HEAD(Window,ids);
+			
+            XRaiseWindow( dpy, windows[0] );
+            if( windows_num > 1 )
+                XRestackWindows( dpy, windows, windows_num );
+            XSync(dpy, False);
+        }
+        raise_scren_panframes (ASDefaultScr);
+        XRaiseWindow(dpy, Scr.ServiceWin);
+	}
+}
+
+void
+publish_aswindow_list( ASWindowList *list, Bool stacking_only )
+{
+	int i ;
+	int stack_len = 0;
+	ASWindow **stack = get_stacking_orger_list(Scr.Windows, &stack_len );
+	ASVector *ids = get_scratch_ids_vector();
+	
+	/* we maybe called from Destroy, in which case one of the clients may be 
+	   delisted from main list, while still present in it's owner's transients list
+	   which is why we use +1 - This was happening for some clients who'd have 
+	   recursive transients ( transient of a transient of a transient ) 
+	   Since we added check to unroll that sequence in tie_aswindow - problem had gone away, 
+	   but lets keep on adding 1 just in case.
+	 */
+	if( !stacking_only ) 
+	{
+		ASBiDirElem *curr = LIST_START(list->clients);
+		while( curr )
+		{
+			ASWindow *asw = (ASWindow*)LISTELEM_DATA(curr);
+			vector_insert_elem(ids, &(asw->w), 1, NULL, False);
+			LIST_GOTO_NEXT(curr);	
+		}	 
+		LOCAL_DEBUG_OUT( "Setting Client List property to include %d windows ", PVECTOR_USED(ids) );
+		set_clients_list (Scr.wmprops, PVECTOR_HEAD(Window,ids), PVECTOR_USED(ids));
+        flush_vector( ids );
+	}		  
+
+	i = stack_len ;
+	while( --i >= 0 )
+		vector_insert_elem(ids, &(stack[i]->w), 1, NULL, False);
+
+	set_stacking_order (Scr.wmprops, PVECTOR_HEAD(Window,ids), PVECTOR_USED(ids));
+}
+
+void
+restack_window_list( int desk )
+{
+	update_stacking_order();
+	send_stacking_order( desk );
+	publish_aswindow_list( Scr.Windows, True );
+	apply_stacking_order( desk );
+}
+
+ASWindow*
+find_topmost_client( int desk, int root_x, int root_y )
+{
+    if( Scr.Windows->clients->count > 0 )
+	{
+		int i ; 
+		int stack_len = 0;
+		ASWindow **stack = get_stacking_orger_list(Scr.Windows, &stack_len );
+
+		for( i = 0 ; i < stack_len ; ++i )
+		{
+           	register ASWindow *asw = stack[i] ;
+			if( ASWIN_DESK(asw) == desk && !ASWIN_GET_FLAGS(asw, AS_Dead) )
+			{
+				register ASCanvas *fc = asw->frame_canvas;  
+				if( fc->root_x <= root_x && fc->root_y <= root_y && 
+					fc->root_x + fc->width + fc->bw*2 > root_x && 
+					fc->root_y + fc->height + fc->bw*2 > root_y ) 
+                    return asw;
+			}
+		}
+	}
+	return NULL ; 
+}
+
+
+
 /*
  * we better have our own routine for changing window stacking order,
  * instead of simply passing it to X server, whenever client request
@@ -1212,7 +1546,61 @@ is_canvas_overlaping (ASCanvas * above, ASCanvas *below)
 	return False ;
 }
 
-#define IS_OVERLAPING(a,b)    is_canvas_overlaping((a)->frame_canvas,(b)->frame_canvas)
+#define IS_OVERLAPING(a,b)    is_canvas_overlaping(a->frame_canvas,b->frame_canvas)
+
+static inline Bool 
+is_overlaping_b( ASWindow *a, ASWindow *b )
+{
+	int i ; 
+	ASWindow **sublist ;
+	if( IS_OVERLAPING(a, b) ) 
+		return True;
+	
+	if( b->transients ) 
+	{
+		sublist = PVECTOR_HEAD(ASWindow*, b->transients);
+		for( i = 0 ; i < PVECTOR_USED(b->transients) ; ++i )
+			if( !ASWIN_GET_FLAGS(sublist[i], AS_Dead) ) 
+				if( IS_OVERLAPING( a, sublist[i] ) ) 
+					return True ;
+	}
+	if( b->group_members ) 
+	{
+		sublist = PVECTOR_HEAD(ASWindow*, b->group_members);
+		for( i = 0 ; i < PVECTOR_USED(b->group_members) ; ++i )
+			if( !ASWIN_GET_FLAGS(sublist[i], AS_Dead) ) 
+				if( IS_OVERLAPING( a, sublist[i] ) ) 
+					return True ;
+	}
+	return False;
+}
+
+static inline Bool 
+is_overlaping( ASWindow *a, ASWindow *b )
+{
+	int i ; 
+	ASWindow **sublist ;
+	if( is_overlaping_b( a, b ) ) 
+		return True ;
+	
+	if( a->transients )
+	{
+		sublist = PVECTOR_HEAD(ASWindow*, a->transients);
+		for( i = 0 ; i < PVECTOR_USED(a->transients) ; ++i )
+			if( !ASWIN_GET_FLAGS(sublist[i], AS_Dead) ) 
+				if( is_overlaping_b( sublist[i], b ) ) 
+					return True ;
+	}
+	if( a->group_members ) 
+	{
+		sublist = PVECTOR_HEAD(ASWindow*, a->group_members);
+		for( i = 0 ; i < PVECTOR_USED(a->group_members) ; ++i )
+			if( !ASWIN_GET_FLAGS(sublist[i], AS_Dead) ) 
+				if( is_overlaping_b( sublist[i], b ) ) 
+					return True ;
+	}
+	return False;	
+}
 
 Bool
 is_window_obscured (ASWindow * above, ASWindow * below)
@@ -1221,7 +1609,7 @@ is_window_obscured (ASWindow * above, ASWindow * below)
     ASWindow **members ;
 
 	if (above != NULL && below != NULL)
-        return IS_OVERLAPING(above, below);
+        return is_overlaping(above, below);
 
 	if (above == NULL && below != NULL)
     {/* checking if window "below" is completely obscured by any of the
@@ -1229,8 +1617,8 @@ is_window_obscured (ASWindow * above, ASWindow * below)
         register int i, end_i ;
 
         l = get_aslayer( ASWIN_LAYER(below), Scr.Windows );
-	if( AS_ASSERT(l) )
-	    return False;
+		if( AS_ASSERT(l) )
+		    return False;
 
         end_i = l->members->used ;
         members = VECTOR_HEAD(ASWindow*,*(l->members));
@@ -1242,7 +1630,7 @@ is_window_obscured (ASWindow * above, ASWindow * below)
 				return False;
             }else if( ASWIN_DESK(t) == ASWIN_DESK(below) )
 			{
-                if (IS_OVERLAPING(t,below))
+                if (is_overlaping(t,below))
 				{
 					return True;
 				}
@@ -1250,12 +1638,13 @@ is_window_obscured (ASWindow * above, ASWindow * below)
         }
     }else if (above != NULL )
     {   /* checking if window "above" is completely obscuring any of the
-           windows with the same layer below it in stacking order */
+           windows with the same layer below it in stacking order, 
+		   or any of its transients !!! */
         register int i ;
 
         l = get_aslayer( ASWIN_LAYER(above), Scr.Windows );
-	if( AS_ASSERT(l) )
-	    return False;
+		if( AS_ASSERT(l) )
+		    return False;
         members = VECTOR_HEAD(ASWindow*,*(l->members));
         for (i = VECTOR_USED(*(l->members))-1 ; i >= 0 ; i-- )
         {
@@ -1263,7 +1652,7 @@ is_window_obscured (ASWindow * above, ASWindow * below)
             if( (t = members[i]) == above )
 				return False;
             else if( ASWIN_DESK(t) == ASWIN_DESK(above) )
-                if (IS_OVERLAPING(above,t) )
+                if (is_overlaping(above,t) )
 					return True;
         }
     }
@@ -1281,6 +1670,12 @@ restack_window( ASWindow *t, Window sibling_window, int stack_mode )
     if( t == NULL )
         return ;
 
+	if( t->transient_owner != NULL )
+		t = t->transient_owner ; 
+		
+	if( ASWIN_GET_FLAGS(t, AS_Dead))
+		return; 
+		
 LOCAL_DEBUG_CALLER_OUT( "%p,%lX,%d", t, sibling_window, stack_mode );
     src_layer = get_aslayer( ASWIN_LAYER(t), Scr.Windows );
 
@@ -1342,13 +1737,29 @@ LOCAL_DEBUG_CALLER_OUT( "%p,%lX,%d", t, sibling_window, stack_mode );
     if( stack_mode != Above && stack_mode != Below )
         sibling = NULL ;
 
-    if( src_layer )
-        vector_remove_elem( src_layer->members, &t );
-    if( dst_layer )
-        vector_insert_elem( dst_layer->members, &t, 1, sibling, above );
+	if( t->group_members  ) 
+	{
+	    int k;
+		int todo = 0;
+	    ASWindow **members = PVECTOR_HEAD(ASWindow*,src_layer->members);
+		ASWindow **group_members = safemalloc( PVECTOR_USED(src_layer->members)*sizeof(ASWindow*));
+	    for( k = 0 ; k < PVECTOR_USED(src_layer->members) ; k++ )
+			if( members[k] != t && members[k]->group_lead == t )
+				group_members[todo++] = members[k];
+		for( k = 0 ; k < todo ; ++k ) 
+		{
+		    vector_remove_elem( src_layer->members, &group_members[k] );
+		    vector_insert_elem( dst_layer->members, &group_members[k], 1, sibling, above );
+			if( sibling ) 
+				sibling = group_members[k];
+		}	
+		free( group_members );
+	}
+    vector_remove_elem( src_layer->members, &t );
+    vector_insert_elem( dst_layer->members, &t, 1, sibling, above );
 
     t->last_restack_time = Scr.last_Timestamp ;
-    restack_window_list( ASWIN_DESK(t), False );
+    restack_window_list( ASWIN_DESK(t) );
 }
 
 
@@ -1396,6 +1807,7 @@ hide_focus()
     if (get_flags(Scr.Feel.flags, ClickToFocus) && Scr.Windows->ungrabbed != NULL)
         grab_aswindow_buttons( Scr.Windows->ungrabbed, False );
 
+	LOCAL_DEBUG_CALLER_OUT( "CHANGE Scr.Windows->focused from %p to NULL", Scr.Windows->focused );
     Scr.Windows->focused = NULL;
     Scr.Windows->ungrabbed = NULL;
     XRaiseWindow(dpy, Scr.ServiceWin);
@@ -1442,6 +1854,8 @@ void autoraise_aswindow( void *data )
 Bool
 focus_window( ASWindow *asw, Window w )
 {
+	LOCAL_DEBUG_CALLER_OUT( "asw = %p, w = %lX", asw, w );
+
   	if( asw != NULL )
         if (get_flags(asw->hints->protocols, AS_DoesWmTakeFocus) && !ASWIN_GET_FLAGS(asw, AS_Dead))
             send_wm_protocol_request (asw->w, _XA_WM_TAKE_FOCUS, Scr.last_Timestamp);
@@ -1458,7 +1872,7 @@ focus_window( ASWindow *asw, Window w )
 	 * focus using their time, as X aready used its own friggin current time.
 	 * Don't ask, its a mess.
 	 * */
-	if( w != None )
+	if( w != None && ASWIN_HFLAGS(asw, AS_AcceptsFocus))
 	{
 		Time t = /*(Scr.Windows->focused == NULL)?CurrentTime:*/Scr.last_Timestamp ;
 		LOCAL_DEBUG_OUT( "XSetInputFocus(window= %lX, time = %lu)", w, t);	  
@@ -1469,14 +1883,31 @@ focus_window( ASWindow *asw, Window w )
     return (w!=None);
 }
 
+void autoraise_window( ASWindow *asw ) 
+{
+    if (Scr.Feel.AutoRaiseDelay == 0)
+    {
+        RaiseWindow( asw );
+    }else if (Scr.Feel.AutoRaiseDelay > 0)
+    {
+        struct timeval tv;
+		LOCAL_DEBUG_OUT( "setting autoraise timer for asw %p", Scr.Windows->focused );
+        gettimeofday (&tv, NULL);
+        Scr.Windows->last_focus_change_sec =  tv.tv_sec;
+        Scr.Windows->last_focus_change_usec = tv.tv_usec;
+        timer_new (Scr.Feel.AutoRaiseDelay, autoraise_aswindow, Scr.Windows->focused);
+    }
+
+}
 
 Bool
-focus_aswindow( ASWindow *asw )
+focus_aswindow( ASWindow *asw, Bool suppress_autoraise )
 {
     Bool          do_hide_focus = False ;
     Bool          do_nothing = False ;
     Window        w = None;
 
+LOCAL_DEBUG_CALLER_OUT( "asw = %p", asw );
     if( asw )
     {
         if (!get_flags( AfterStepState, ASS_WarpingMode) )
@@ -1492,7 +1923,7 @@ focus_aswindow( ASWindow *asw )
                         (ASWIN_GET_FLAGS( asw, AS_Iconic ) &&
                             asw->icon_canvas == NULL && asw->icon_title_canvas == NULL );
 
-        if( !ASWIN_HFLAGS(asw, AS_AcceptsFocus) )
+        if( !ASWIN_FOCUSABLE(asw))
         {
             if( Scr.Windows->focused != NULL && ASWIN_DESK(Scr.Windows->focused) == Scr.CurrentDesk )
                 do_nothing = True ;
@@ -1550,22 +1981,12 @@ focus_aswindow( ASWindow *asw )
         show_warning( "unable to focus window %lX that is about to be unmapped for client %lX, frame %lX", w, asw->w, asw->frame );
     else
     {
-
 		focus_window( asw, w );
 
+		LOCAL_DEBUG_CALLER_OUT( "CHANGE Scr.Windows->focused from %p to %p", Scr.Windows->focused, asw );
         Scr.Windows->focused = asw ;
-        if (Scr.Feel.AutoRaiseDelay == 0)
-        {
-            RaiseWindow( asw );
-        }else if (Scr.Feel.AutoRaiseDelay > 0)
-        {
-            struct timeval tv;
-
-            gettimeofday (&tv, NULL);
-            Scr.Windows->last_focus_change_sec =  tv.tv_sec;
-            Scr.Windows->last_focus_change_usec = tv.tv_usec;
-            timer_new (Scr.Feel.AutoRaiseDelay, autoraise_aswindow, Scr.Windows->focused);
-        }
+		if( !suppress_autoraise ) 
+			autoraise_window( asw ); 
     }
 
     XSync(dpy, False );
@@ -1581,14 +2002,14 @@ Bool
 focus_active_window()
 {
     /* don't fiddle with focus if we are in housekeeping mode !!! */
-LOCAL_DEBUG_OUT( "checking if we are in housekeeping mode (%ld)", get_flags(AfterStepState, ASS_HousekeepingMode) );
+LOCAL_DEBUG_CALLER_OUT( "checking if we are in housekeeping mode (%ld)", get_flags(AfterStepState, ASS_HousekeepingMode) );
     if( get_flags(AfterStepState, ASS_HousekeepingMode) || Scr.Windows->active == NULL )
         return False ;
 
     if( Scr.Windows->focused == Scr.Windows->active )
         return True ;                          /* already has focus */
 
-    return focus_aswindow( Scr.Windows->active );
+    return focus_aswindow( Scr.Windows->active, FOCUS_ASW_CAN_AUTORAISE );
 }
 
 /* second version of above : */
@@ -1714,6 +2135,23 @@ ASWindow2func_data( FunctionCode func, ASWindow *asw, FunctionData *fdata, char
     fdata->hotkey = (*scut);
 }
 
+struct ASSortMenu_Aux
+{
+	FunctionData fdata ; 
+	void *ref_data ; 
+};
+
+int
+compare_menu_func_data_name(const void *a, const void *b) 
+{
+	struct ASSortMenu_Aux *aa = *(struct ASSortMenu_Aux **)a ;
+	struct ASSortMenu_Aux *ab = *(struct ASSortMenu_Aux **)b ;
+
+/*	LOCAL_DEBUG_OUT( "aa = %p, ab = %p", aa, ab ); */
+	return strcmp(aa->fdata.name ? aa->fdata.name : "", ab->fdata.name ? ab->fdata.name : "");
+}
+
+
 MenuData *
 make_desk_winlist_menu(  ASWindowList *list, int desk, int sort_order, Bool icon_name )
 {
@@ -1746,24 +2184,26 @@ make_desk_winlist_menu(  ASWindowList *list, int desk, int sort_order, Bool icon
 
 	if( sort_order == ASO_Alpha ) 
 	{
-		FunctionData **menuitems = safecalloc(sizeof(FunctionData *), max_i);
+		struct ASSortMenu_Aux **menuitems = safecalloc(max_i, sizeof(struct ASSortMenu_Aux *));
 		int numitems = 0;
         
 		for( i = 0 ; i < max_i ; ++i ) 
 		{
             if ((ASWIN_DESK(clients[i]) == desk || !IsValidDesk(desk)) && !ASWIN_HFLAGS(clients[i], AS_SkipWinList)) 
 			{
-				menuitems[numitems] = safecalloc(1, sizeof(FunctionData));
-				ASWindow2func_data( F_RAISE_IT, clients[i], menuitems[numitems], &scut, icon_name ); 
+				menuitems[numitems] = safecalloc(1, sizeof(struct ASSortMenu_Aux));
+/*				LOCAL_DEBUG_OUT( "menuitems[%d] = %p", numitems, menuitems[numitems] ); */
+				menuitems[numitems]->ref_data = clients[i];
+				ASWindow2func_data( F_RAISE_IT, clients[i], &(menuitems[numitems]->fdata), &scut, icon_name ); 
 				++numitems;
             }
         }
-		qsort(menuitems, numitems, sizeof(FunctionData *), compare_func_data_name);
+		qsort(menuitems, numitems, sizeof(struct ASSortMenu_Aux *), compare_menu_func_data_name);
 		for( i = 0 ; i < numitems ; ++i ) 
 		{
-			if( (mdi = add_menu_fdata_item( md, menuitems[i], NULL, 
+			if( (mdi = add_menu_fdata_item( md, &(menuitems[i]->fdata), NULL, 
 								 get_flags( Scr.Feel.flags, WinListHideIcons) ? NULL : 
-								 get_client_icon_image(ASDefaultScr, clients[i]->hints))) != NULL )
+								 get_client_icon_image(ASDefaultScr, ((ASWindow*)(menuitems[i]->ref_data))->hints))) != NULL )
 				set_flags( mdi->flags, MD_ScaleMinipixmapDown );
 			safefree(menuitems[i]); /* scrubba-dub-dub */
 		}
