@@ -36,6 +36,7 @@
 
 #include <limits.h>
 #include <signal.h>
+#include <unistd.h>
 
 #include "../../libAfterStep/session.h"
 #include "../../libAfterStep/moveresize.h"
@@ -797,7 +798,7 @@ void moveresize_func_handler( FunctionData *data, ASEvent *event, int module )
 	int height = asw->status->height ;
 
 	new_val1 = APPLY_VALUE_UNIT(Scr.MyDisplayWidth,data->func_val[0],data->unit_val[0]);
-	LOCAL_DEBUG_OUT( "val1 = %ld, unit1 = %ld, new_val1 = %d", data->func_val[0],data->unit_val[0], new_val1 );
+	LOCAL_DEBUG_OUT( "val1 = %d, unit1 = %d, new_val1 = %d", (int)data->func_val[0], (int)data->unit_val[0], new_val1 );
 	new_val2 = APPLY_VALUE_UNIT(Scr.MyDisplayHeight,data->func_val[1],data->unit_val[1]);
 	if( data->func == F_MOVE )
 	{
@@ -971,8 +972,8 @@ void iconify_func_handler( FunctionData *data, ASEvent *event, int module )
 	if( event->client )
 	{
 
-LOCAL_DEBUG_CALLER_OUT( "function %ld (val0 = %ld), event %d, window 0x%lX, window_name \"%s\", module %d",
-			data?data->func:0, data?data->func_val[0]:0, event?event->x.type:-1, event?(unsigned long)event->w:0, event->client?ASWIN_NAME(event->client):"none", module );
+LOCAL_DEBUG_CALLER_OUT( "function %d (val0 = %d), event %d, window 0x%lX, window_name \"%s\", module %d",
+			(int)(data?data->func:0), (int)(data?data->func_val[0]:0), event?event->x.type:-1, event?(unsigned long)event->w:0, event->client?ASWIN_NAME(event->client):"none", module );
 	if (ASWIN_GET_FLAGS(event->client, AS_Iconic) )
 	{
 		if (data->func_val[0] <= 0)
@@ -1151,13 +1152,9 @@ void restart_func_handler( FunctionData *data, ASEvent *event, int module )
 
 void exec_func_handler( FunctionData *data, ASEvent *event, int module )
 {
-    XGrabPointer( dpy, Scr.Root, True,
-			      ButtonPressMask | ButtonReleaseMask,
-		  GrabModeAsync, GrabModeAsync, Scr.Root, Scr.Feel.cursors[ASCUR_Wait], CurrentTime);
-    XSync (dpy, 0);
-    spawn_child( data->text, -1, -1, None, C_NO_CONTEXT, True, False, NULL );
-    XUngrabPointer (dpy, CurrentTime);
-    XSync (dpy, 0);
+	/* no sense in grabbing Pointer here as fork will return rather fast not creating 
+	   much of the delay */
+    spawn_child( data->text, -1, -1, NULL, None, C_NO_CONTEXT, True, False, NULL );
 }
 
 int find_escaped_chr_pos( const char *str, char c )
@@ -1264,18 +1261,63 @@ void exec_in_term_func_handler( FunctionData *data, ASEvent *event, int module )
 		if( full_cmdl ) 
 		{	
 			LOCAL_DEBUG_OUT( "full_cmdl = [%s]", full_cmdl );
-    		XGrabPointer( dpy, Scr.Root, True,
-			      		ButtonPressMask | ButtonReleaseMask,
-		  		GrabModeAsync, GrabModeAsync, Scr.Root, Scr.Feel.cursors[ASCUR_Wait], CurrentTime);
-    		XSync (dpy, 0);
-    		spawn_child( full_cmdl, -1, -1, None, C_NO_CONTEXT, True, False, NULL );
-    		XUngrabPointer (dpy, CurrentTime);
+			/* no sense in grabbing Pointer here as fork will return rather fast not creating 
+	   		   much of the delay */
+    		spawn_child( full_cmdl, -1, -1, NULL, None, C_NO_CONTEXT, True, False, NULL );
 			free( full_cmdl );
 		}
-    	XSync (dpy, 0);
 	}
 }
 
+int   /* return -1 - undetermined, 0 - no term, 1 - terminal requiresd */
+check_tool_needs_term(const char *tool)
+{
+	static char *known_text_mode_apps[] = {
+		"vi", "vim", "joe", "jed", "ne", "le", "moe", "elvis", "jove", "dav", "aoeui", 
+		"elinks", "lynx", 
+		NULL 
+	};
+	ASDesktopEntry *de ;
+	
+	char *name = NULL;
+	struct stat   st;
+	int res = -1;
+	int i;
+
+	parse_file_name(tool, NULL, &name);
+	LOCAL_DEBUG_OUT( "checking if \"%s\" requires term ...", name );
+	de = fetch_desktop_entry( CombinedCategories, name );
+	if (de!= NULL)
+	{
+		LOCAL_DEBUG_OUT( "DesktopEntry \"%s\" found", de->Name);
+		res = get_flags(de->flags, ASDE_Terminal)?1:0;
+	}else if (lstat (tool, &st) != -1) 
+	{	/* now we need to resolve all the symlinking */
+		
+		if ((st.st_mode & S_IFMT) == S_IFLNK)
+		{
+			char linkdst[1024];
+			int len = readlink(tool,linkdst, sizeof(linkdst)-1);
+			if ( len > 0)
+			{
+				linkdst[len] = '\0';
+				res = check_tool_needs_term(linkdst);
+				LOCAL_DEBUG_OUT( "checking symlink target \"%s\" - res = %d", linkdst, res);
+			}
+		}
+	}
+	
+	/* last resort - check hardcoded list */	
+	for( i = 0 ; res < 0 && known_text_mode_apps[i] ; ++i )
+		if (strcmp(name, known_text_mode_apps[i]) == 0 )
+			res = 1;
+
+	LOCAL_DEBUG_OUT( "\"%s\" requires term  = %d", name, res );
+
+	free (name);
+	return res;
+}
+
 void exec_tool_func_handler( FunctionData *data, ASEvent *event, int module )
 {
 	ASToolType tool = (data->func == F_ExecBrowser)?ASTool_Browser:ASTool_Editor ;
@@ -1284,14 +1326,20 @@ void exec_tool_func_handler( FunctionData *data, ASEvent *event, int module )
 		char *full_cmdl = safemalloc( strlen(Environment->tool_command[tool]) +1+strlen(data->text)+1 );
 		sprintf( full_cmdl, "%s %s", Environment->tool_command[tool], data->text );
 		LOCAL_DEBUG_OUT( "full_cmdl = [%s]", full_cmdl );
-   		XGrabPointer(  dpy, Scr.Root, True,
-			      		ButtonPressMask | ButtonReleaseMask,
-				  		GrabModeAsync, GrabModeAsync, Scr.Root, Scr.Feel.cursors[ASCUR_Wait], CurrentTime);
-    	XSync (dpy, 0);
-    	spawn_child( full_cmdl, -1, -1, None, C_NO_CONTEXT, True, False, NULL );
-    	XUngrabPointer (dpy, CurrentTime);
+		
+		if (check_tool_needs_term(Environment->tool_command[tool]) > 0)
+		{
+			char * old_text = data->text;
+			data->text = full_cmdl;
+			exec_in_term_func_handler(data, event, module);			
+			data->text = old_text;
+		}else
+		{
+			/* no sense in grabbing Pointer here as fork will return rather fast not creating 
+	   		   much of the delay */
+    		spawn_child( full_cmdl, -1, -1, NULL, None, C_NO_CONTEXT, True, False, NULL );
+		}
 		free( full_cmdl );
-    	XSync (dpy, 0);
 	}
 }
 
@@ -1704,7 +1752,7 @@ void goto_page_func_handler( FunctionData *data, ASEvent *event, int module )
 #ifndef NO_VIRTUAL
     int newvx = data->func_val[0]*event->scr->MyDisplayWidth;
     int newvy = data->func_val[1]*event->scr->MyDisplayHeight;
-LOCAL_DEBUG_OUT( "val(%ld,%ld)->scr(%d,%d)->newv(%d,%d)", data->func_val[0], data->func_val[1], event->scr->MyDisplayWidth, event->scr->MyDisplayHeight, newvx, newvy );
+LOCAL_DEBUG_OUT( "val(%d,%d)->scr(%d,%d)->newv(%d,%d)", (int)data->func_val[0], (int)data->func_val[1], event->scr->MyDisplayWidth, event->scr->MyDisplayHeight, newvx, newvy );
     MoveViewport ( newvx, newvy, True);
 #endif
 }
@@ -1728,14 +1776,8 @@ void gethelp_func_handler( FunctionData *data, ASEvent *event, int module )
 		if (ASWIN_RES_NAME(event->client)!= NULL)
 		{
 		char         *realfilename = PutHome(HELPCOMMAND);
-	    XGrabPointer (dpy, Scr.Root, True,
-			  ButtonPressMask | ButtonReleaseMask,
-			  GrabModeAsync, GrabModeAsync, Scr.Root, Scr.Feel.cursors[ASCUR_Wait], CurrentTime);
-	    XSync (dpy, 0);
-	    spawn_child( realfilename, -1, -1, None, C_NO_CONTEXT, True, False, ASWIN_RES_NAME(event->client), NULL);
+	    spawn_child( realfilename, -1, -1, NULL, None, C_NO_CONTEXT, True, False, ASWIN_RES_NAME(event->client), NULL);
 	    free (realfilename);
-	    XUngrabPointer (dpy, CurrentTime);
-	    XSync (dpy, 0);
 	}
 }
 
