@@ -89,7 +89,7 @@ CreateMenuData (char *name)
     if( Scr.Feel.Popups == NULL )
         init_list_of_menus(&(Scr.Feel.Popups), True);
     md = new_menu_data( Scr.Feel.Popups, name );
-	md->recent_items = Scr.Feel.recent_submenu_items ;
+	md->recent_items = -1 ; /* use Scr.Feel.recent_submenu_items by default! */
 	return md;
 }
 
@@ -169,6 +169,7 @@ ParseBody (void *data, FILE * fd, Bool (*item_func)(void *, const char*))
 	buf = safemalloc (MAXLINELENGTH + 1);
 	while ((ptr = fgets (buf, MAXLINELENGTH, fd)) != NULL)
 	{
+		LOCAL_DEBUG_OUT( "parsing Popup line \"%s\"", buf );
 		while (isspace (*ptr))
 			ptr++;
         if (!item_func(data, ptr))
@@ -345,18 +346,161 @@ add_minipixmap_from_dirtree_item( dirtree_t * tree, MenuData *menu )
 		menu_data_item_from_func (menu, fdata, False);
 }	 
 
+char *
+add_menuitem_submenu_from_file( FILE *fp2, char *buf, char *title )
+{
+	MenuData *md = NULL ; 
+	long start ;
+	static int submenu_id = DIRTREE_ID ; 
+	char *name = string_from_int( ++submenu_id );
+
+	md = CreateMenuData( name );
+	start = ftell( fp2 );
+	LOCAL_DEBUG_OUT( "parsing Popup \"%s\"", name );
+	
+	if( !ParseBody( md, fp2, MenuDataItemParse ) )
+	{
+		LOCAL_DEBUG_OUT( "parsing Popup body failed.. cleaning up%s", "" );
+		remove_hash_item( Scr.Feel.Popups, AS_HASHABLE(name), NULL, True );
+		/* delete menu and restore the file pointer */
+		fseek( fp2, start, SEEK_SET );
+		free( name );
+		name = NULL ; 
+	} else if( title != NULL ) 
+	{
+		if( md->first == NULL || md->first->fdata->func != F_TITLE ) 
+		{
+			FunctionData *fdata = create_named_function( F_TITLE, title);
+			/* We exploit that scan_for_hotkey removes & (marking hotkey) from name */
+			scan_for_hotkey (fdata->name);
+		    menu_data_item_from_func (md, fdata, False);
+		}
+	
+	}
+	
+	return name;
+}
+
+
+int
+add_menuitem_from_file( FILE *fp2, char *buf, dirtree_t *t, Bool show_unavailable, MenuData *menu ) 
+{
+	int lines_read = 0 ;
+	Bool available = True ;
+	char *name = NULL ;
+	FunctionData *fdata = NULL ;
+	FunctionData *valid_func = NULL ; 
+	FunctionData *minipixmap = NULL ;
+	if( fp2 == NULL ) 
+		return 0;
+	/* try to load a command */
+LOCAL_DEBUG_OUT( "fp2=%p, buf=%p, t=%p", fp2, buf, t);
+
+	while( fgets (buf, MAXLINELENGTH, fp2) != NULL)
+	{
+		int parse_err ; 
+		++lines_read;
+LOCAL_DEBUG_OUT( "%s", buf );
+		if( fdata == NULL ) 
+			fdata = create_named_function(F_EXEC, NULL);
+		parse_err = parse_func (buf, fdata, True);
+		if ( parse_err > FUNC_ERR_START && parse_err < 0 ) /* data is actuall shell command line */
+			fdata->text = stripcpy (buf);
+			
+		if( fdata->name == NULL )
+		    fdata->name = mystrdup( t->stripped_name );
+		
+		if( fdata->func == F_POPUP && fdata->text == NULL ) 
+			fdata->text = add_menuitem_submenu_from_file( fp2, buf, fdata->name );
+
+#ifndef NO_AVAILABILITYCHECK
+		available = check_fdata_availability( fdata );
+		if( !available )
+			fdata->func = F_NOP;
+#endif					
+		if( IsMinipixmapFunc(fdata->func) )
+		{
+			if( minipixmap == NULL ) 
+			{
+				minipixmap = fdata ;
+				fdata = NULL ; 
+			}
+		}else if( fdata->func != F_NOP ) 
+		{
+			if( valid_func == NULL ) 
+			{
+				valid_func = fdata ; 
+				fdata = NULL ; 
+			}					
+		}
+
+		if( fdata ) 
+		{
+			if( name == NULL && fdata->name != NULL ) 
+			{
+				name = fdata->name ;
+				fdata->name = NULL ;
+			}
+			destroy_func_data( &fdata );
+		}
+	}
+	if( available || show_unavailable)
+	{	
+		if( valid_func  ) 
+			menu_data_item_from_func (menu, valid_func, False);
+		else
+		{
+			fdata = create_named_function(F_NOP, name?name:t->stripped_name);
+	  		fdata->text = mystrdup (t->name);
+	        menu_data_item_from_func (menu, fdata, False);
+		}
+		if( minipixmap ) 
+			menu_data_item_from_func (menu, minipixmap, False);
+		else if( t->icon != NULL )
+		{
+			fdata = create_named_function(F_MINIPIXMAP, t->icon);
+	        menu_data_item_from_func (menu, fdata, False);
+		}
+	}else
+	{
+		if( valid_func )
+			destroy_func_data(&valid_func);
+		if( minipixmap )
+			destroy_func_data(&minipixmap);
+	}
+	if( name ) 
+		free( name );
+	return lines_read ;
+}
+
+static void update_menu_item_from_dirtree( MenuDataItem *menu_item, dirtree_t *t )
+{
+	if( get_flags( t->flags, DIRTREE_NAME_IS_UTF8 ) )
+		set_flags( menu_item->flags, MD_NameIsUTF8 );
+	if( t->Comment ) 
+	{
+		menu_item->comment = interpret_ascii_string( t->Comment );
+		if( get_flags( t->flags, DIRTREE_COMMENT_IS_UTF8 ) )
+			set_flags( menu_item->flags, MD_CommentIsUTF8 );
+	}
+}
+
 MenuData     *
 dirtree_make_menu2 (dirtree_t * tree, char *buf, Bool reload_submenus)
 {
 /*  extern struct config* func_config; */
 	dirtree_t    *t;
     MenuData     *menu;
+    MenuDataItem    *menu_item;
 	FunctionData *fdata;
 
 	/* make self */
 	if (tree->flags & DIRTREE_KEEPNAME)
+	{
         menu = CreateMenuData (tree->name);
-	else
+		if( get_flags( tree->flags, DIRTREE_NAME_IS_UTF8 ) )
+			set_flags( menu->flags, MD_NameIsUTF8 );
+	}else
 	{
 		sprintf (buf, "%d", tree->flags & DIRTREE_ID);
         menu = CreateMenuData (buf);
@@ -368,8 +512,8 @@ dirtree_make_menu2 (dirtree_t * tree, char *buf, Bool reload_submenus)
 	fdata = create_named_function( F_TITLE, tree->name);
 	/* We exploit that scan_for_hotkey removes & (marking hotkey) from name */
 	scan_for_hotkey (fdata->name);
-    menu_data_item_from_func (menu, fdata, False);
-
+    menu_item = menu_data_item_from_func (menu, fdata, False);
+	update_menu_item_from_dirtree( menu_item, tree ); 
 	add_minipixmap_from_dirtree_item( tree, menu );
 
 	for (t = tree->child; t != NULL; t = t->next)
@@ -379,40 +523,32 @@ dirtree_make_menu2 (dirtree_t * tree, char *buf, Bool reload_submenus)
 		if (t->flags & DIRTREE_DIR)
 		{
 /************* Creating Popup Title entry : ************************/
-			fdata = create_named_function(F_POPUP, t->stripped_name);
+			fdata = create_named_function((t->child==NULL)?F_NOP:F_POPUP, t->stripped_name);
             if( reload_submenus )
                 fdata->popup = dirtree_make_menu2 (t, buf, reload_submenus);
             else
                 fdata->popup = FindPopup( t->stripped_name, True );
-/*            if( fdata->popup == NULL )
-                fdata->func = F_NOP;
-*/
+
 			fdata->hotkey = scan_for_hotkey (fdata->name);
 			if (t->flags & DIRTREE_KEEPNAME)
 				fdata->text = mystrdup (t->stripped_name);
 			else
 				fdata->text = string_from_int (t->flags & DIRTREE_ID);
 
-            menu_data_item_from_func (menu, fdata, False);
-
+            menu_item = menu_data_item_from_func (menu, fdata, False);
+			update_menu_item_from_dirtree( menu_item, t); 
  			add_minipixmap_from_dirtree_item( t, menu );
 /************* Done creating Popup Title entry : ************************/
 		} else if( t->de ) 
 		{
 			if( t->de->type == ASDE_TypeApplication ) 
 			{	
-				FunctionCode func = F_EXEC ;
-				if( get_flags( t->de->flags, ASDE_Unavailable ) )
-					func = F_NOP ;
-				if( get_flags( t->de->flags, ASDE_Terminal ) )
-					func = F_ExecInTerm;	   
-				else if( get_flags( t->de->flags, ASDE_ASModule ) )
-					func = F_MODULE;	   
-		 			
-				fdata = create_named_function(func, t->stripped_name);	   
-            	fdata->text = mystrdup( t->de->clean_exec );
-            	menu_data_item_from_func (menu, fdata, False);
- 				add_minipixmap_from_dirtree_item( t, menu );
+				fdata = desktop_entry2function( t->de, t->stripped_name);
+				LOCAL_DEBUG_OUT( "adding \"%s\" with function %d", t->stripped_name, fdata->func );
+
+            	menu_item = menu_data_item_from_func (menu, fdata, False);
+				update_menu_item_from_dirtree( menu_item, t ); 
+				add_minipixmap_from_dirtree_item( t, menu );
 			}
 		}else if (t->command.func != F_NOP)
 		{
@@ -439,92 +575,18 @@ dirtree_make_menu2 (dirtree_t * tree, char *buf, Bool reload_submenus)
 				destroy_func_data( &fdata );
 			}else
 			{
-	            menu_data_item_from_func (menu, fdata, False);
+	            menu_item = menu_data_item_from_func (menu, fdata, False);
 	 			add_minipixmap_from_dirtree_item( t, menu );
 			}
         } else
 		{
 			FILE *fp2 = fopen (t->path, "r");
-			int lines_read = 0 ;
-
+			int lines_read = add_menuitem_from_file( fp2, buf, t, get_flags(tree->flags, DIRTREE_SHOW_UNAVAILABLE), menu );
+if( fp2 == NULL ) 
+{
+LOCAL_DEBUG_OUT( "failed to open \"%s\"", t->path?t->path:"(NULL)" );
+}
 			fdata = NULL ; 
-			if( fp2 != NULL  )
-			{
-				Bool available = True ;
-				char *name = NULL ;
-				FunctionData *valid_func = NULL ; 
-				FunctionData *minipixmap = NULL ;
-				/* try to load a command */
-				while( fgets (buf, MAXLINELENGTH, fp2) != NULL)
-				{
-					int parse_err ; 
-					++lines_read;
-					if( fdata == NULL ) 
-						fdata = create_named_function(F_EXEC, NULL);
-					parse_err = parse_func (buf, fdata, True);
-					if ( parse_err < 0 && parse_err > FUNC_ERR_START ) /* data is actuall shell command line */
-						fdata->text = stripcpy (buf);
-					if( fdata->name == NULL )
-		                fdata->name = mystrdup( t->stripped_name );
-#ifndef NO_AVAILABILITYCHECK
-					available = check_fdata_availability( fdata );
-					if( !available )
-						fdata->func = F_NOP;
-#endif					
-					if( IsMinipixmapFunc(fdata->func) )
-					{
-						if( minipixmap == NULL ) 
-						{
-							minipixmap = fdata ;
-							fdata = NULL ; 
-						}
-					}else if( fdata->func != F_NOP ) 
-					{
-						if( valid_func == NULL ) 
-						{
-							valid_func = fdata ; 
-							fdata = NULL ; 
-						}					
-					}
-					
-					if( fdata ) 
-					{
-						if( name == NULL && fdata->name != NULL ) 
-						{
-							name = fdata->name ;
-							fdata->name = NULL ;
-						}
-						destroy_func_data( &fdata );
-					}
-				}
-				if( available || get_flags(tree->flags, DIRTREE_SHOW_UNAVAILABLE))
-				{	
-					if( valid_func  ) 
-						menu_data_item_from_func (menu, valid_func, False);
-					else
-					{
-						fdata = create_named_function(F_NOP, name?name:t->stripped_name);
-	  					fdata->text = mystrdup (t->name);
-	          			menu_data_item_from_func (menu, fdata, False);
-					}
-					if( minipixmap ) 
-						menu_data_item_from_func (menu, minipixmap, False);
-					else if( t->icon != NULL )
-					{
-						fdata = create_named_function(F_MINIPIXMAP, t->icon);
-	          			menu_data_item_from_func (menu, fdata, False);
-					}
-				}else
-				{
-					if( valid_func )
-						destroy_func_data(&valid_func);
-					if( minipixmap )
-						destroy_func_data(&minipixmap);
-				}
-				if( name ) 
-					free( name );
-			}
-			
 			if( fp2 == NULL || lines_read == 0 )
 			{
 				Bool available = True ;
